#!/usr/bin/env python
"""
_Subscription_

A simple object representing a Subscription in WMBS.

A subscription is just a way to link many sets of jobs to a
fileset and track the process of the associated jobs. It is
associated to a single fileset and a single workflow.
"""
from __future__ import print_function

from future.utils import listvalues

import logging
from collections import Counter

from WMCore.DataStructs.Fileset import Fileset as WMFileset
from WMCore.DataStructs.Subscription import Subscription as WMSubscription
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.WMBSBase import WMBSBase
from WMCore.WMBS.Workflow import Workflow


class Subscription(WMBSBase, WMSubscription):
    """
    WMBS Subscription

    WMBS object for executing a task or similar chunk of work
    """

    def __init__(self, fileset=None, workflow=None, id=-1,
                 split_algo="FileBased", type="Processing"):
        WMBSBase.__init__(self)

        # If a fileset or workflow isn't passed in the base class will create
        # empty non-WMBS filesets and workflows.  We want WMBS filesets and
        # workflows so we'll create those here.
        if fileset is None:
            fileset = Fileset()
        if workflow is None:
            workflow = Workflow()

        WMSubscription.__init__(self, fileset=fileset, workflow=workflow,
                                split_algo=split_algo, type=type)

        self.setdefault("id", id)

        self.bulkDeleteLimit = 500
        return

    def create(self):
        """
        Add the subscription to the database
        """
        if self.exists() is not False:
            self.load()
            return

        existingTransaction = self.beginTransaction()
        action = self.daofactory(classname="Subscriptions.New")
        action.execute(fileset=self["fileset"].id, type=self["type"],
                       split_algo=self["split_algo"],
                       workflow=self["workflow"].id,
                       conn=self.getDBConn(),
                       transaction=self.existingTransaction())

        self.load()
        self.commitTransaction(existingTransaction)
        return

    def exists(self):
        """
        See if the subscription is in the database
        """
        action = self.daofactory(classname="Subscriptions.Exists")
        result = action.execute(fileset=self["fileset"].id,
                                workflow=self["workflow"].id,
                                conn=self.getDBConn(),
                                transaction=self.existingTransaction())
        return result

    def load(self):
        """
        _load_

        Load any meta data about the subscription.  This include the id, type,
        split algorithm, fileset id and workflow id.  Either the subscription id
        or the fileset id and workflow id must be specified for this to work.
        """
        existingTransaction = self.beginTransaction()

        if self["id"] > 0:
            action = self.daofactory(classname="Subscriptions.LoadFromID")
            result = action.execute(id=self["id"],
                                    conn=self.getDBConn(),
                                    transaction=self.existingTransaction())
        else:
            action = self.daofactory(classname="Subscriptions.LoadFromFilesetWorkflow")
            result = action.execute(fileset=self["fileset"].id,
                                    workflow=self["workflow"].id,
                                    conn=self.getDBConn(),
                                    transaction=self.existingTransaction())

        self["type"] = result["type"]
        self["id"] = result["id"]
        self["split_algo"] = result["split_algo"]

        # Only load the fileset and workflow if they haven't been loaded
        # already.
        if self["fileset"].id < 0:
            self["fileset"] = Fileset(id=result["fileset"])

        if self["workflow"].id < 0:
            self["workflow"] = Workflow(id=result["workflow"])

        self.commitTransaction(existingTransaction)
        return

    def loadData(self):
        """
        _loadData_

        Load all data having to do with the subscription including all the
        files contained in the fileset and the workflow meta data.
        """
        existingTransaction = self.beginTransaction()

        if self["id"] < 0 or self["fileset"].id < 0 or self["workflow"].id < 0:
            self.load()

        self["fileset"].loadData()
        self["workflow"].load()

        self.commitTransaction(existingTransaction)
        return

    def addWhiteBlackList(self, sites):
        """
        _addWhiteBlackList_

        Add a site white or black list for this transaction.  The sites
        paremeter must be a list of dictionaries with the following keys:
          site_name - The CMS name of the site
          valid - A bool, True for a white list, False for a black list.
        """
        existingTransaction = self.beginTransaction()

        for site in sites:
            site["sub"] = self["id"]

        action = self.daofactory(classname="Subscriptions.AddValidation")
        dummyResult = action.execute(sites=sites, conn=self.getDBConn(), transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)

    def getWhiteBlackList(self):
        """
        _getWhiteBlackList_

        Retrieve the white lists and black lists for this subscription.
        """
        existingTransaction = self.beginTransaction()

        action = self.daofactory(classname="Subscriptions.GetValidation")
        result = action.execute(self["id"],
                                conn=self.getDBConn(),
                                transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)
        return result

    def filesOfStatus(self, status, loadChecksums=True, doingJobSplitting=False):
        """
        _filesOfStatus_

        Return a Set of File objects that have the given status with respect
        to this subscription.
        """
        existingTransaction = self.beginTransaction()

        status = status.title()
        files = set()
        action = self.daofactory(classname="Subscriptions.Get%sFiles" % status)
        fileList = action.execute(self["id"], conn=self.getDBConn(),
                                  transaction=self.existingTransaction())

        if doingJobSplitting:
            fileInfoAct = self.daofactory(classname="Files.GetForJobSplittingByID")
        else:
            fileInfoAct = self.daofactory(classname="Files.GetByID")

        fileInfoDict = fileInfoAct.execute(file=[x["file"] for x in fileList],
                                           conn=self.getDBConn(),
                                           transaction=self.existingTransaction())

        # Run through all files
        for f in fileList:
            fl = File(id=f['file'])
            if loadChecksums:
                fl.loadChecksum()
            fl.update(fileInfoDict[f['file']])
            if 'locations' in f:
                fl.setLocation(f['locations'], immediateSave=False)
            files.add(fl)

        self.commitTransaction(existingTransaction)
        return files

    def acquireFiles(self, files=None):
        """
        _acquireFiles_

        Mark all files objects that are passed in as acquired for this
        subscription.  If no files are passed in then all available files
        will be acquired.
        """
        existingTransaction = self.beginTransaction()

        action = self.daofactory(classname="Subscriptions.AcquireFiles")

        if not files:
            files = self.filesOfStatus("Available")
        elif isinstance(files, (Fileset, WMFileset)):
            pass
        else:
            files = self.makelist(files)

        if len(files) == 0:
            self.commitTransaction(existingTransaction)
            return

        action.execute(self['id'], file=[x["id"] for x in files],
                       conn=self.getDBConn(),
                       transaction=self.existingTransaction())

        try:
            self.commitTransaction(existingTransaction)
        except Exception as ex:
            print("Found exception %s" % ex)
            logging.error("Exception found in committing acquireFiles transaction: %s", ex)
        return

    def completeFiles(self, files):
        """
        Mark a (set of) file(s) as completed.
        """
        existingTransaction = self.beginTransaction()

        files = self.makelist(files)

        completeAction = self.daofactory(classname="Subscriptions.CompleteFiles")
        completeAction.execute(subscription=self["id"],
                               file=[x["id"] for x in files],
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)
        return

    def failFiles(self, files):
        """
        Mark a (set of) file(s) as failed.
        """
        existingTransaction = self.beginTransaction()

        files = self.makelist(files)

        failAction = self.daofactory(classname="Subscriptions.FailFiles")
        failAction.execute(subscription=self["id"],
                           file=[x["id"] for x in files],
                           conn=self.getDBConn(),
                           transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)
        return

    def getJobs(self):
        """
        Return a list of all the jobs associated with a subscription
        """
        jobsAction = self.daofactory(classname="Subscriptions.Jobs")
        jobs = jobsAction.execute(subscription=self["id"],
                                  conn=self.getDBConn(),
                                  transaction=self.existingTransaction())

        return jobs

    def delete(self):
        """
        _delete_

        Delete this subscription from the database.
        """
        action = self.daofactory(classname="Subscriptions.Delete")
        action.execute(id=self["id"], conn=self.getDBConn(),
                       transaction=self.existingTransaction())

        return

    def isCompleteOnRun(self, runID):
        """
        _isCompleteOnRun_

        Check all the files in the given subscripton and the given run are completed.

        To: check query whether performance can be improved
        """
        statusAction = self.daofactory(classname="Subscriptions.IsCompleteOnRun")
        fileCount = statusAction.execute(self["id"], runID,
                                         conn=self.getDBConn(),
                                         transaction=self.existingTransaction())

        if fileCount == 0:
            return True
        else:
            return False

    def getNumberOfJobsPerSite(self, location, state):
        """
        _getNumberOfJobsPerSite_

        Access the number of jobs at a site in a given status for a given subscription
        """
        jobLocate = self.daofactory(classname="Subscriptions.GetNumberOfJobsPerSite")

        result = listvalues(jobLocate.execute(location=location,
                                              subscription=self['id'],
                                              state=state))[0]
        return result

    def getJobGroups(self):
        """
        _getJobGroups_

        Returns a list of job group IDs associated with the subscription with new jobs
        """
        action = self.daofactory(classname="Subscriptions.GetJobGroups")
        return action.execute(self['id'], conn=self.getDBConn(),
                              transaction=self.existingTransaction())

    def getAllJobGroups(self):
        """
        _getAllJobGroups_

        Returns a list of ALL jobGroups associated with the subscription
        """
        action = self.daofactory(classname="Subscriptions.GetAllJobGroups")
        return action.execute(self['id'], conn=self.getDBConn(),
                              transaction=self.existingTransaction())

    def deleteEverything(self):
        """
        _deleteEverything_

        This function should delete the subscription, and absolutely everything else having anything
        to do with the subscription that is NOT in use by any other piece.  It should check for all
        the proper ownerships through a sequence of DAO calls that will take forever.

        Nothing except the taskArchiver should be calling this.
        """
        existingTransaction = self.beginTransaction()

        jobGroups = self.getAllJobGroups()
        filesets = []

        # The order here is important
        # You need to delete files and filesets from the bottom up
        # In order to not violate parentage

        # Get output filesets from jobGroups
        for jobGroupID in jobGroups:
            loadAction = self.daofactory(classname="JobGroup.LoadFromID")
            result = loadAction.execute(jobGroupID, conn=self.getDBConn(),
                                        transaction=self.existingTransaction())
            filesets.append(result['output'])

        # Get output filesets from the workflow
        for entry in self['workflow'].outputMap:
            for outputFilesets in self['workflow'].outputMap[entry]:
                wid = outputFilesets["output_fileset"].id
                if wid not in filesets:
                    filesets.append(wid)

        # Do the input fileset LAST!
        filesets.append(self['fileset'].id)

        self.commitTransaction(existingTransaction)

        # First, jobs
        # If there are too many jobs, delete them in separate
        # transactions to reduce database load
        deleteAction = self.daofactory(classname="Jobs.Delete")
        jobDeleteList = []
        for job in self.getJobs():
            jobDeleteList.append(job['id'])
        if len(jobDeleteList) > 0:
            if len(jobDeleteList) <= self.bulkDeleteLimit:
                existingTransaction = self.beginTransaction()
                deleteAction.execute(id=jobDeleteList, conn=self.getDBConn(),
                                     transaction=self.existingTransaction())
                self.commitTransaction(existingTransaction)
            else:
                while len(jobDeleteList) > 0:
                    existingTransaction = self.beginTransaction()
                    toDelete = jobDeleteList[:self.bulkDeleteLimit]
                    jobDeleteList = jobDeleteList[self.bulkDeleteLimit:]
                    deleteAction.execute(id=toDelete, conn=self.getDBConn(),
                                         transaction=self.existingTransaction())

                    self.commitTransaction(existingTransaction)

        # Next jobGroups
        deleteAction = self.daofactory(classname="JobGroup.Delete")
        existingTransaction = self.beginTransaction()
        for jobGroupID in jobGroups:
            deleteAction.execute(id=jobGroupID, conn=self.getDBConn(),
                                 transaction=self.existingTransaction())
        self.commitTransaction(existingTransaction)

        # Now, get the filesets that needs to be deleted
        action = self.daofactory(classname="Fileset.CheckForDelete")
        existingTransaction = self.beginTransaction()
        results = action.execute(fileids=filesets,
                                 subid=self['id'],
                                 conn=self.getDBConn(),
                                 transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)

        deleteFilesets = [x['id'] for x in results]

        # Delete files in sets
        # Each set of files deleted in a separate transaction
        for filesetID in deleteFilesets:
            fileset = Fileset(id=filesetID)

            # Load the files
            filesetFiles = []
            action = self.daofactory(classname="Files.InFileset")
            results = action.execute(fileset=filesetID,
                                     conn=self.getDBConn(),
                                     transaction=self.existingTransaction())

            for result in results:
                filesetFiles.append(result['fileid'])

            # Now get rid of unused files
            if len(filesetFiles) < 1:
                # if we have unused files, of course
                continue

            parent = self.daofactory(classname="Files.DeleteParentCheck")
            action = self.daofactory(classname="Files.DeleteCheck")

            if len(filesetFiles) <= self.bulkDeleteLimit:
                existingTransaction = self.beginTransaction()
                parent.execute(file=filesetFiles, fileset=fileset.id,
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction())
                action.execute(file=filesetFiles, fileset=fileset.id,
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction())
                self.commitTransaction(existingTransaction)
            else:
                while len(filesetFiles) > 0:
                    existingTransaction = self.beginTransaction()
                    toDelete = filesetFiles[:self.bulkDeleteLimit]
                    filesetFiles = filesetFiles[self.bulkDeleteLimit:]
                    parent.execute(file=toDelete, fileset=fileset.id,
                                   conn=self.getDBConn(),
                                   transaction=self.existingTransaction())
                    action.execute(file=toDelete, fileset=fileset.id,
                                   conn=self.getDBConn(),
                                   transaction=self.existingTransaction())
                    self.commitTransaction(existingTransaction)

        # Start a new transaction for filesets, workflow, and the subscription
        existingTransaction = self.beginTransaction()
        for filesetID in deleteFilesets:
            # Now actually delete the filesets
            action = self.daofactory(classname="Fileset.DeleteCheck")
            dummyDeleteFilesets = action.execute(fileid=filesetID, subid=self['id'],
                                                 conn=self.getDBConn(), transaction=self.existingTransaction())

        # Next Workflow
        action = self.daofactory(classname="Workflow.DeleteCheck")
        action.execute(workid=self["workflow"].id, subid=self["id"],
                       conn=self.getDBConn(),
                       transaction=self.existingTransaction())

        self.delete()
        self.commitTransaction(existingTransaction)
        return

    def isFileCompleted(self, files):
        """
        _isFileCompleted_

        Returns True if all the given files are in complete status
        Return False if one of files are not in complete status
        """
        if not isinstance(files, list):
            files = [files]

        action = self.daofactory(classname="Subscriptions.GetCompletedByFileList")
        fileIDs = action.execute(self['id'], files, conn=self.getDBConn(),
                                 transaction=self.existingTransaction())

        for f in files:
            if f['id'] not in fileIDs:
                return False

        return True

    def markFinished(self, finished=True):
        """
        _markFinished_

        Sets the finished status of the subscription
        to the given value
        """

        action = self.daofactory(classname="Subscriptions.MarkFinishedSubscriptions")
        action.execute(self['id'], conn=self.getDBConn(),
                       transaction=self.existingTransaction())
        self.commitTransaction(self.existingTransaction)

        return

    def bulkCommit(self, jobGroups):
        """
        _bulkCommit_

        Commits all objects created during job splitting.  This is dangerous because it assumes
        that you can pass in all jobGroups.
        """

        jobList = []
        jobGroupList = []
        nameList = []

        # You have to do things in this order:
        # 1) First create Filesets, then jobGroups
        # 2) Second, create jobs pointing to jobGroups
        # 3) Deal with masks, etc.

        # First, do we exist?  We better
        # This happens in its own transaction
        if self['id'] == -1:
            self.create()

        existingTransaction = self.beginTransaction()

        # You need to create a number of Filesets equal to the
        # number of jobGroups.

        for _ in jobGroups:
            # Make a random name for each fileset
            nameList.append(makeUUID())

        # Create filesets
        action = self.daofactory(classname="Fileset.BulkNewReturn")
        fsIDs = action.execute(nameList=nameList, open=True,
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction())

        for jobGroup in jobGroups:
            jobGroup.uid = makeUUID()
            jobGroupList.append({'subscription': self['id'],
                                 'uid': jobGroup.uid,
                                 'output': fsIDs.pop()})

        action = self.daofactory(classname="JobGroup.BulkNewReturn")
        jgIDs = action.execute(bulkInput=jobGroupList,
                               conn=self.getDBConn(),
                               transaction=self.existingTransaction())

        for jobGroup in jobGroups:
            for idUID in jgIDs:
                # This should assign an ID to the right job
                if jobGroup.uid == idUID['guid']:
                    jobGroup.id = idUID['id']
                    break

        for jobGroup in jobGroups:
            for job in jobGroup.newjobs:
                if job["id"] is not None:
                    continue

                job["jobgroup"] = jobGroup.id

                if job["name"] is None:
                    job["name"] = makeUUID()
                jobList.append(job)

        bulkAction = self.daofactory(classname="Jobs.New")
        result = bulkAction.execute(jobList=jobList, conn=self.getDBConn(),
                                    transaction=self.existingTransaction())

        # Move jobs to jobs from newjobs
        for jobGroup in jobGroups:
            jobGroup.jobs.extend(jobGroup.newjobs)
            jobGroup.newjobs = []

        # Use the results of the bulk commit to get the jobIDs
        fileDict = {}
        jobFileRunLumis = []
        for job in jobList:
            job['id'] = result[job['name']]
            fileDict[job['id']] = []
            for f in job['input_files']:
                fileDict[job['id']].append(f['id'])
                fileMask = job['mask'].filterRunLumisByMask(runs=f['runs'])
                for runObj in fileMask:
                    run = runObj.run
                    lumis = runObj.lumis
                    for lumi in lumis:
                        jobFileRunLumis.append((job['id'], f['id'], run, lumi))

        # Create a list of mask binds
        maskList = []
        for job in jobList:
            mask = job['mask']
            if len(list(mask['runAndLumis'].keys())) > 0:
                # Then we have multiple binds
                binds = mask.produceCommitBinds(jobID=job['id'])
                maskList.extend(binds)
            else:
                mask['jobID'] = job['id']
                maskList.append(mask)

        maskAction = self.daofactory(classname="Masks.Save")
        maskAction.execute(jobid=None, mask=maskList, conn=self.getDBConn(),
                           transaction=self.existingTransaction())

        fileAction = self.daofactory(classname="Jobs.AddFiles")
        fileAction.execute(jobDict=fileDict, conn=self.getDBConn(),
                           transaction=self.existingTransaction())

        # wfid = self['workflow'].id
        # Add work units and associate them
        # wuAction = self.daofactory(classname='WorkUnit.Add')
        # wufAction = self.daofactory(classname='Jobs.AddWorkUnits')

        # Make a count of how many times each job appears in the list of jobFileRunLumis
        # jobUnitCounts = Counter([jid for jid, _, _, _ in jobFileRunLumis])

        # for jid, fid, run, lumi in jobFileRunLumis:
        #     wuAction.execute(taskid=wfid, fileid=fid, run=run, lumi=lumi, last_unit_count=jobUnitCounts[jid],
        #                      conn=self.getDBConn(), transaction=self.existingTransaction())
        # wufAction.execute(jobFileRunLumis=jobFileRunLumis,
        #                   conn=self.getDBConn(), transaction=self.existingTransaction())

        fileList = []
        for job in jobList:
            fileList.extend(job['input_files'])

        self.acquireFiles(files=fileList)
        self.commitTransaction(existingTransaction)
        return



# -*- coding: utf-8 -*-
"""
_GetPendingSlots_

Oracle implementation of Locations.GetPendingSlots

Created on Mon Jun 18 13:51:29 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.GetPendingSlots import GetPendingSlots as MySQLGetPendingSlots

class GetPendingSlots(MySQLGetPendingSlots):
    """
    Same as MySQL version

    """
    pass



#!/usr/bin/env python


from WMCore.Services.DBS.ProdException import ProdException


def formatEx(excepInst):
    """
    _formatEx_

    given a DbdException instance, generate a simple message from it
    """
    msg = "%s:%s %s" % (excepInst.__class__.__name__,
                        excepInst.getErrorMessage(),
                        excepInst.getErrorCode(),
                        )
    return msg

def formatEx3(excepInst):
    """
    _formatEx_

    given a DbdException instance, generate a simple message from it
    """
    msg = "%s:%s" % (excepInst.__class__.__name__, str(excepInst))
    return msg



class DataMgmtError(ProdException):
    """
    _DataMgmtError_

    General Exception from DataMgmt Interface

    """
    def __init__(self, message, errorNo = 1000 , **data):
        ProdException.__init__(self, message, errorNo, **data)

class DBSWriterError(DataMgmtError):
    """
    _DBSWriterError_

    Generic Exception for DBS Write Error

    """
    def __init__(self, msg, **data):
        DataMgmtError.__init__(self, msg, 1001, **data)



class DBSReaderError(DataMgmtError):
    """
    _DBSReaderError_

    Generic Exception for DBS Read Error

    """
    def __init__(self, msg, **data):
        DataMgmtError.__init__(self, msg, 1002, **data)



#!/usr/bin/env python

"""
__PerformanceMonitor_

Monitor object which checks the job to ensure it is working inside
the agreed limits of virtual memory and wallclock time, and terminate it
if it exceeds them.
"""
from __future__ import division

import logging
import os
import os.path
import signal
import time

import WMCore.Algorithms.SubprocessAlgos as subprocessAlgos
import WMCore.FwkJobReport.Report as Report
from WMCore.WMException import WMException
from WMCore.WMRuntime.Monitors.WMRuntimeMonitor import WMRuntimeMonitor
from WMCore.WMSpec.Steps.Executor import getStepSpace
from WMCore.WMSpec.WMStep import WMStepHelper

getStepName = lambda step: WMStepHelper(step).name()


def getStepPID(stepSpace, stepName):
    """
    _getStepPID_

    Find the PID for a step given its stepSpace from the file
    """
    currDir = stepSpace.location
    pidFile = os.path.join(currDir, 'process.id')
    if not os.path.isfile(pidFile):
        msg = "Could not find process ID file for step %s" % stepName
        logging.error(msg)
        return None

    with open(pidFile,'r') as filehandle:
        output = filehandle.read()

    try:
        stepPID = int(output)
    except ValueError:
        msg = "Couldn't find a number"
        logging.error(msg)
        return None

    return stepPID


def average(numbers):
    """
    Quick averaging function

    """
    return float(sum(numbers)) / len(numbers)


class PerformanceMonitorException(WMException):
    """
    _PerformanceMonitorException_

    Just a performance monitor error...nothing to see here
    """
    pass


class PerformanceMonitor(WMRuntimeMonitor):
    """
    _PerformanceMonitor_

    Monitors the performance by pinging ps and
    recording data regarding the current step
    """

    def __init__(self):
        """
        Actual variable initialization
        in initMonitor
        """

        self.pid = None
        self.uid = os.getuid()
        self.monitorBase = "ps -p %i -o pid,ppid,rss,pcpu,pmem,cmd -ww | grep %i"
        self.pssMemoryCommand = "awk '/^Pss/ {pss += $2} END {print pss}' /proc/%i/smaps"
        self.monitorCommand = None
        self.currentStepSpace = None
        self.currentStepName = None

        self.rss = []
        self.pcpu = []
        self.pmem = []

        self.maxPSS = None
        self.softTimeout = None
        self.hardTimeout = None
        self.logPath = None
        self.startTime = None
        self.killRetry = False  # will trigger a hard (SIGTERM) instead of soft kill

        self.watchStepTypes = []

        self.disableStep = False

        WMRuntimeMonitor.__init__(self)

        return

    def initMonitor(self, task, job, logPath, args=None):
        """
        _initMonitor_

        Puts together the information needed for the monitoring
        to actually find everything.
        """
        args = args or {}

        # Set the steps we want to watch
        self.watchStepTypes = args.get('WatchStepTypes', ['CMSSW', 'PerfTest'])

        self.maxPSS = args.get('maxPSS', args.get('maxRSS'))
        self.softTimeout = args.get('softTimeout', None)
        self.hardTimeout = args.get('hardTimeout', None)
        self.numOfCores = args.get('cores', None)

        self.logPath = os.path.join(logPath)

        return

    def jobStart(self, task):
        """
        _jobStart_

        Acknowledge that the job has started and initialize the time
        """
        self.startTime = time.time()

        return

    def stepStart(self, step):
        """
        _stepStart_

        Assure that the monitor is pointing at the right step
        """

        self.stepHelper = WMStepHelper(step)
        self.currentStepName = getStepName(step)
        self.currentStepSpace = None

        if not self.stepHelper.stepType() in self.watchStepTypes:
            self.disableStep = True
            logging.debug("PerformanceMonitor ignoring step of type %s", self.stepHelper.stepType())
            return
        else:
            logging.debug("Beginning PeformanceMonitor step Initialization")
            self.disableStep = False

        return

    def stepEnd(self, step, stepReport):
        """
        _stepEnd_

        Package the information and send it off
        """

        if not self.disableStep:
            # No information to correlate
            return

        self.currentStepName = None
        self.currentStepSpace = None

        return

    def periodicUpdate(self):
        """
        Run on the defined intervals.

        """
        killProc = False
        killHard = False
        reason = ''
        errorCodeLookup = {'PSS': 50660,
                           'Wallclock time': 50664,
                           '': 99999}

        if self.disableStep:
            # Then we aren't doing CPU monitoring
            # on this step
            return

        if self.currentStepName is None:
            # We're between steps
            return

        if self.currentStepSpace is None:
            # Then build the step space
            self.currentStepSpace = getStepSpace(self.stepHelper.name())

        stepPID = getStepPID(self.currentStepSpace, self.currentStepName)

        if stepPID is None:
            # Then we have no step PID, we can do nothing
            return

        # Now we run the ps monitor command and collate the data
        # Gathers RSS, %CPU and %MEM statistics from ps
        ps_cmd = self.monitorBase % (stepPID, stepPID)
        stdout, _stderr, _retcode = subprocessAlgos.runCommand(ps_cmd)

        ps_output = stdout.split()
        if not len(ps_output) > 6:
            # Then something went wrong in getting the ps data
            msg = "Error when grabbing output from process ps\n"
            msg += "output = %s\n" % ps_output
            msg += "command = %s\n" % ps_cmd
            logging.error(msg)
            return

        # run the command to gather PSS memory statistics from /proc/<pid>/smaps
        smaps_cmd = self.pssMemoryCommand % (stepPID)
        stdout, _stderr, _retcode = subprocessAlgos.runCommand(smaps_cmd)

        smaps_output = stdout.split()
        if not len(smaps_output) == 1:
            # Then something went wrong in getting the smaps data
            msg = "Error when grabbing output from smaps\n"
            msg += "output = %s\n" % smaps_output
            msg += "command = %s\n" % smaps_cmd
            logging.error(msg)
            return

        # smaps also returns data in kiloBytes, let's make it megaBytes
        # I'm also confused with these megabytes and mebibytes...
        pss = int(smaps_output[0]) // 1000

        logging.info("PSS: %s; RSS: %s; PCPU: %s; PMEM: %s", smaps_output[0], ps_output[2], ps_output[3], ps_output[4])

        msg = 'Error in CMSSW step %s\n' % self.currentStepName
        msg += 'Number of Cores: %s\n' % self.numOfCores

        if self.maxPSS is not None and pss >= self.maxPSS:
            msg += "Job has exceeded maxPSS: %s MB\n" % self.maxPSS
            msg += "Job has PSS: %s MB\n" % pss
            killProc = True
            reason = 'PSS'
        elif self.hardTimeout is not None and self.softTimeout is not None:
            currentTime = time.time()
            if (currentTime - self.startTime) > self.softTimeout:
                killProc = True
                reason = 'Wallclock time'
                msg += "Job has been running for more than: %s\n" % str(self.softTimeout)
                msg += "Job has been running for: %s\n" % str(currentTime - self.startTime)
            if (currentTime - self.startTime) > self.hardTimeout:
                killHard = True
                msg += "Job exceeded soft timeout"

        if not killProc:
            # then job is behaving well, there is nothing to do
            return

        # make sure we persist the performance error only once
        if not self.killRetry:
            logging.error(msg)
            report = Report.Report()
            # Find the global report
            logPath = os.path.join(self.currentStepSpace.location,
                                   '../../../',
                                   os.path.basename(self.logPath))
            try:
                if os.path.isfile(logPath):
                    # We should be able to find existant job report.
                    # If not, we're in trouble
                    logging.debug("Found pre-existant error report in PerformanceMonitor termination.")
                    report.load(logPath)
                # Create a new step that won't be overridden by an exiting CMSSW
                if not report.retrieveStep(step="PerformanceError"):
                    report.addStep(reportname="PerformanceError")
                report.addError(stepName="PerformanceError", exitCode=errorCodeLookup[reason],
                                errorType="PerformanceKill", errorDetails=msg)
                report.save(logPath)
            except Exception as ex:
                # Basically, we can't write a log report and we're hosed
                # Kill anyway, and hope the logging file gets written out
                msg2 = "Exception while writing out jobReport.\n"
                msg2 += "Aborting job anyway: unlikely you'll get any error report.\n"
                msg2 += "Error: %s" % str(ex)
                logging.exception(msg2)

        try:
            if not killHard and not self.killRetry:
                logging.error("Attempting to kill step using SIGUSR2")
                os.kill(stepPID, signal.SIGUSR2)
            else:
                logging.error("Attempting to kill step using SIGTERM")
                os.kill(stepPID, signal.SIGTERM)
        except Exception:
            logging.error("Attempting to kill step using SIGTERM")
            os.kill(stepPID, signal.SIGTERM)
        finally:
            self.killRetry = True

        return



#!/usr/bin/env python
"""
_GetInjectedWorkflows_

MySQL implementation of Workflow.GetInjectedWorkflows

"""
__all__ = []

from WMCore.Database.DBFormatter import DBFormatter

class GetInjectedWorkflows(DBFormatter):
    """
    Gets workflows that have been fully injected into WMBS
    """

    sql = """SELECT DISTINCT name FROM wmbs_workflow WHERE injected = :injected"""


    def execute(self, injected = False, conn = None, transaction = False):
        """
        _execute_

        Simple query that looks for either injected or uninjected files
        """

        if not injected:
            binds = {'injected': 0}
        else:
            binds = {'injected': 1}

        result = self.dbi.processData(self.sql, binds, conn = conn,
                                      transaction = transaction)

        dResult = self.formatDict(result)
        finalResult = []

        for entry in dResult:
            finalResult.append(entry['name'])

        return finalResult



#!/usr/bin/env python
"""
Unittests for the Services/Rucio/RucioUtils module
"""
import logging
import unittest

from WMCore.Services.Rucio.RucioUtils import (RUCIO_VALID_PROJECT, validateMetaData,
                                              isTapeRSE, dropTapeRSEs, weightedChoice)


class RucioUtilsTest(unittest.TestCase):
    """
    Test class for the RucioUtils module
    """

    def setUp(self):
        """
        Setup for unit tests
        """
        self.logger = logging.getLogger()

    def tearDown(self):
        """
        Nothing to be done for this case
        """

    def testMetaDataValidation(self):
        """
        Test the `validateMetaData` validation function
        """
        for thisProj in RUCIO_VALID_PROJECT:
            response = validateMetaData("any_DID_name", dict(project=thisProj), self.logger)
            self.assertTrue(response)

        # test with no "project" meta data at all
        response = validateMetaData("any_DID_name", {}, self.logger)
        self.assertTrue(response)

        # now an invalid "project" meta data
        response = validateMetaData("any_DID_name", {"project": "mistake"}, self.logger)
        self.assertFalse(response)

    def testWeightedChoice(self):
        """
        Test the `weightedChoice` utilitarian function
        """
        pickDistr = {"CERN": 0, "FNAL": 0, "KIT": 0}
        rsesWithApproval = []
        rsesWeight = []
        for idx, rseName in enumerate(pickDistr):
            rsesWithApproval.append((rseName, False))
            rsesWeight.append((idx * 10) + 10)

        for _ in range(11):
            rse = weightedChoice(rsesWithApproval, rsesWeight)[0]
            pickDistr[rse] += 1
        # it's not a reproducible distribution, so just assume
        # that CERN is barely selected
        self.assertTrue(pickDistr["CERN"] <= 2)

    def testIsTapeRSE(self):
        """
        Test the `isTapeRSE` utilitarian function
        """
        self.assertTrue(isTapeRSE("T1_US_FNAL_Tape"))
        self.assertFalse(isTapeRSE("T1_US_FNAL_Disk"))
        self.assertFalse(isTapeRSE("T1_US_FNAL_Disk_Test"))
        self.assertFalse(isTapeRSE("T1_US_FNAL_Tape_Test"))
        self.assertFalse(isTapeRSE(""))

    def testDropTapeRSEs(self):
        """
        Test the `dropTapeRSEs` utilitarian function
        """
        tapeOnly = ["T1_US_FNAL_Tape", "T1_ES_PIC_Tape"]
        diskOnly = ["T1_US_FNAL_Disk", "T1_US_FNAL_Disk_Test", "T2_CH_CERN"]
        mixed = ["T1_US_FNAL_Tape", "T1_US_FNAL_Disk", "T1_US_FNAL_Disk_Test", "T1_ES_PIC_Tape"]
        self.assertCountEqual(dropTapeRSEs(tapeOnly), [])
        self.assertCountEqual(dropTapeRSEs(diskOnly), diskOnly)
        self.assertCountEqual(dropTapeRSEs(mixed), ["T1_US_FNAL_Disk", "T1_US_FNAL_Disk_Test"])


if __name__ == '__main__':
    unittest.main()



"""
_Block_

A dictionary based object meant to represent a row in the active_block/block table.
"""




class Block(dict):
    """
    _Block_

    A dictionary based object meant to represent subset of dbs block.
    Which will just need for workQueue update.
    It contains the following keys:
      Name
      Size
      NumEvent
      NumFiles
    """
    def __init__(self, **args):
        """
        ___init___

        Initialize all attributes.
        """
        dict.__init__(self)

        self.setdefault("Name", None)
        self.setdefault("Size", None)
        self.setdefault("NumEvents", None)
        self.setdefault("NumFiles", None)
        self.update(args)

    @staticmethod
    def getBlock(blockInfo):
        """
        convert to the Block structure from db column format
        """
        block = Block()

        block["Name"] = blockInfo[0]["name"]
        block["NumFiles"] = blockInfo[0]["num_files"]
        block["NumEvents"] = blockInfo[0]["num_events"]
        block["Size"] = blockInfo[0]["block_size"]

        return block



#!/usr/bin/env python
"""
    WorkQueue tests
"""
from __future__ import absolute_import

import tempfile
import unittest
import cProfile
import pstats
from WMQuality.Emulators.WMSpecGenerator.WMSpecGenerator import WMSpecGenerator
from WMCore.WorkQueue.WorkQueue import globalQueue
from .WorkQueueTestCase import WorkQueueTestCase


class WorkQueueProfileTest(WorkQueueTestCase):
    """
    _WorkQueueTest_

    """

    def setUp(self):
        """
        If we dont have a wmspec file create one

        Warning: For the real profiling test including
        spec generation. need to use real spec instead of
        using emulator generated spec which doesn't include
        couchDB access and cmssw access
        """
        WorkQueueTestCase.setUp(self)

        self.cacheDir = tempfile.mkdtemp()
        self.specGenerator = WMSpecGenerator(self.cacheDir)
        self.specNamePrefix = "TestReReco_"
        self.specs = self.createReRecoSpec(5, "file")
        # Create queues
        self.globalQueue = globalQueue(DbName=self.globalQDB,
                                       InboxDbName=self.globalQInboxDB,
                                       NegotiationTimeout=0)

    def tearDown(self):
        """tearDown"""
        WorkQueueTestCase.tearDown(self)
        try:
            self.specGenerator.removeSpecs()
        except Exception:
            pass

    def createReRecoSpec(self, numOfSpec, kind="spec"):
        specs = []
        for i in range(numOfSpec):
            specName = "%s%s" % (self.specNamePrefix, (i + 1))
            specs.append(self.specGenerator.createReRecoSpec(specName, kind))
        return specs

    def createProfile(self, name, function):
        filename = name
        prof = cProfile.Profile()
        prof.runcall(function)
        prof.dump_stats(filename)
        p = pstats.Stats(filename)
        p.strip_dirs().sort_stats('cumulative').print_stats(0.1)
        p.strip_dirs().sort_stats('time').print_stats(0.1)
        p.strip_dirs().sort_stats('calls').print_stats(0.1)
        # p.strip_dirs().sort_stats('name').print_stats(10)

    def testQueueElementProfile(self):
        self.createProfile('queueElementProfile.prof',
                           self.multipleQueueWorkCall)

    def multipleQueueWorkCall(self):
        i = 0
        for wmspec in self.specs:
            i += 1
            self.globalQueue.queueWork(wmspec, self.specNamePrefix + str(i), 'test_team')


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_GetNumberOfJobsPerWorkflow_

MySQL implementation of Jobs.GetNumberOfJobsPerWorkflow
"""

__all__ = []



import logging

from WMCore.Database.DBFormatter import DBFormatter

class GetNumberOfJobsPerWorkflow(DBFormatter):
    """
    Exactly what it says on the tin

    """

    sql = """SELECT COUNT(*) FROM wmbs_job
               INNER JOIN wmbs_jobgroup ON wmbs_job.jobgroup = wmbs_jobgroup.id
               INNER JOIN wmbs_subscription ON wmbs_subscription.id = wmbs_jobgroup.subscription
               WHERE wmbs_subscription.workflow = :workflow
               """


    def execute(self, workflow, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given job ID and then format and return
        the result.
        """

        binds = {'workflow': workflow}

        result = self.dbi.processData(self.sql, binds, conn = conn, transaction = transaction)

        return self.format(result)[0][0]  #Should only be one result



import logging

class TestRegistryFile1:


    def doSomething(self, parameters = {}):
        logging.debug("TestCollection_1.TestRegistryFile1 is doing something")
        logging.debug("With parameters: "+str(parameters))
        return parameters



import unittest
import os

from WMCore.Configuration import loadConfigurationFile

class ConfigurationTest(unittest.TestCase):
    """
        primitive test for all example configuration in WMCore/etc
        Currently it is just testing syntax errors on configurations
        by importing the configuration.
        To do: extend to test semantic errors if necessary.

    """
    def setUp(self):
        self.configBase = "%s/etc" % os.environ["WMCORE_ROOT"]

    def testGlobalWorkQueueConfig(self):
        config = "%s/GlobalWorkQueueConfig.py" % self.configBase
        configObj = loadConfigurationFile(config)

    def testEmulaterConfig(self):
        config = "%s/EmulatorConfig.py" % self.configBase
        configObj = loadConfigurationFile(config)

    def testWMAgentConfig(self):
        config = "%s/WMAgentConfig.py" % self.configBase
        configObj = loadConfigurationFile(config)

    def testWMAgentConfigAgentOnly(self):
        config = "%s/WMAgentConfigAgentOnly.py" % self.configBase
        configObj = loadConfigurationFile(config)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_Load_

Oracle implementation of Masks.Load
"""

__all__ = []



from WMCore.WMBS.MySQL.Masks.Load import Load as LoadMasksMySQL

class Load(LoadMasksMySQL):
    sql = LoadMasksMySQL.sql



#!/usr/bin/env python
"""
_Workflow_

Unittest for the WMCore.DataStructs.Workflow class.
"""

import unittest

from WMCore.DataStructs.Workflow import Workflow
from WMCore.DataStructs.Fileset import Fileset

class WorkflowTest(unittest.TestCase):
    """
    _WorkflowTest_

    """
    def testDefinition(self):
        """
        Tests to make sure Workflow is defined correctly

        """

        testSpec  = "test"
        testOwner = "mnorman"
        testName  = "testName"

        testWorkflow = Workflow(spec = testSpec, owner = testOwner, name = testName)

        self.assertEqual(testWorkflow.spec,  testSpec)
        self.assertEqual(testWorkflow.owner, testOwner)
        self.assertEqual(testWorkflow.name,  testName)

        return

    def testAddOutput(self):
        """
        _testAddOutput_

        Tests the addOutput functionality of the DataStructs Workflow.
        """
        filesetA = Fileset(name = "filesetA")
        filesetB = Fileset(name = "filesetB")
        filesetC = Fileset(name = "filesetC")

        testWorkflow = Workflow(spec = "test", owner = "mnorman")
        testWorkflow.addOutput("out1", filesetA, filesetB)
        testWorkflow.addOutput("out1", filesetB, filesetA)
        testWorkflow.addOutput("out2", filesetC)

        self.assertEqual(len(testWorkflow.outputMap["out1"]), 2,
                         "Error: There should be two mappings for out1.")
        self.assertEqual(len(testWorkflow.outputMap["out2"]), 1,
                         "Error: There should be two mappings for out2.")

        self.assertTrue({"output_fileset": filesetA,
                         "merged_output_fileset": filesetB} in testWorkflow.outputMap["out1"],
                        "Error: Fileset A should be in the output map.")
        self.assertTrue({"output_fileset": filesetB,
                         "merged_output_fileset": filesetA} in testWorkflow.outputMap["out1"],
                        "Error: Fileset B should be in the output map.")

        self.assertEqual(filesetC, testWorkflow.outputMap["out2"][0]["output_fileset"],
                        "Error: Fileset C should be in the output map.")
        self.assertEqual(None, testWorkflow.outputMap["out2"][0]["merged_output_fileset"],
                         "Error: The merged output should be None.")
        return

if __name__ == '__main__':
    unittest.main()



"""
_ContinuousSummaryHistogram_t_

Unit test module for the ContinuousSummaryHistogram module.
It checks the sanity of the histogram constructions,
corner cases and its compatibility with JSON data published
to couch.

Created on Nov 20, 2012

@author: dballest
"""
from __future__ import division
from builtins import range
from future.utils import viewvalues

import os
import unittest
import random

from WMCore.DataStructs.MathStructs.ContinuousSummaryHistogram import ContinuousSummaryHistogram
from WMCore.Database.CMSCouch import Database
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit

class ContinuousSummaryHistogramTest(unittest.TestCase):

    def setUp(self):
        """
        _setUp_

        Setup a couch database for testing
        of produced JSON
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setupCouch("histogram_dump_t")
        random.seed()
        self.histogramDB = Database(url=os.getenv("COUCHURL"), dbname="histogram_dump_t")

    def tearDown(self):
        """
        _tearDown_

        Clean the couch
        """
        self.testInit.tearDownCouch()

    def buildRandomNumberList(self, n, distribution = "normalvariate", **kwargs):
        """
        _buildRandomNumberList_

        Builds a list with n pseudorandomly distributed
        numbers according to some given distribution
        """
        numberList = []
        if not kwargs:
            kwargs = {"mu" : 0, "sigma" : 1}
        for _ in range(n):
            generator = getattr(random, distribution)
            numberList.append(generator(**kwargs))

        return numberList

    def testA_BasicTest(self):
        """
        _testA_BasicTest_

        Build a histogram from a set of uniformly
        distributed pseudorandom numbers. Check
        that the statistic properties
        in the histogram are accurate to some degree,
        that the histogram binning is done right and
        that this can become a document an uploaded to couch
        """
        inputData = self.buildRandomNumberList(1000)

        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel')

        # Populate the histogram
        for point in inputData:
            histogram.addPoint(point)

        # Get the JSON
        jsonHistogram = histogram.toJSON()

        # Check the histogram core data
        self.assertEqual(jsonHistogram["title"], "TestHisto")
        self.assertEqual(jsonHistogram["xLabel"], "MyLabel")
        self.assertAlmostEqual(jsonHistogram["average"], 0.0, places = 0)
        self.assertAlmostEqual(jsonHistogram["stdDev"], 1.0, places = 0)
        self.assertEqual(len(jsonHistogram["data"]), 16)
        self.assertTrue(jsonHistogram["continuous"])

        # Check the internal data
        self.assertEqual(jsonHistogram["internalData"]["yLabel"], "SomeoneElsesLabel")
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 1000)

        # Try to commit it to couch
        jsonHistogram["_id"] = jsonHistogram["title"]
        self.histogramDB.commitOne(jsonHistogram)

        storedJSON = self.histogramDB.document("TestHisto")
        self.assertEqual(len(storedJSON["data"]), 16)

        return

    def testB_extremeData(self):
        """
        _testB_extremeData_

        Put extreme points in the data and try to build a histogram.
        Check that it can process all this correctly
        """

        # First no data
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel')
        jsonHistogram = histogram.toJSON()
        self.assertEqual(jsonHistogram["title"], "TestHisto")
        self.assertEqual(jsonHistogram["xLabel"], "MyLabel")
        self.assertEqual(jsonHistogram["average"], 0.0)
        self.assertEqual(jsonHistogram["stdDev"], 0.0)
        self.assertEqual(len(jsonHistogram["data"]), 0)

        # Data with NaNs and Infs
        inputData = self.buildRandomNumberList(100)
        inputData.append(float('NaN'))
        inputData.append(float('Inf'))
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel')
        for point in inputData:
            histogram.addPoint(point)
        jsonHistogram = histogram.toJSON()
        self.assertAlmostEqual(jsonHistogram["average"], 0.0, places = 0)
        self.assertAlmostEqual(jsonHistogram["stdDev"], 1.0, places = 0)
        self.assertEqual(len(jsonHistogram["data"]), 7)
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 100)

        # One single point, P5
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel')
        histogram.addPoint(5)
        jsonHistogram = histogram.toJSON()
        self.assertEqual(jsonHistogram["average"], 5.0)
        self.assertEqual(jsonHistogram["stdDev"], 0.0)
        self.assertEqual(len(jsonHistogram["data"]), 1)
        self.assertEqual(jsonHistogram["data"]["5.0,5.0"], 1)
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 1)

        # Test that toJSON is idempotent
        inputData = self.buildRandomNumberList(100)
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel')
        for point in inputData:
            histogram.addPoint(point)
        jsonHistogram = histogram.toJSON()
        oldData = jsonHistogram["data"]
        jsonHistogram = histogram.toJSON()
        self.assertAlmostEqual(jsonHistogram["average"], 0.0, places = 0)
        self.assertAlmostEqual(jsonHistogram["stdDev"], 1.0, places = 0)
        self.assertEqual(len(jsonHistogram["data"]), 7)
        self.assertEqual(jsonHistogram["data"], oldData)
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 100)

        return

    def testC_compactHistogram(self):
        """
        _testC_compactHistogram_

        Check that we can create smaller histograms objects
        by chopping outliers and dropping the data all together
        """

        # Input normally distributed data and chop anything above 1 stdev (32% of data)
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel',
                                               dropOutliers = True, sigmaLimit = 1)
        inputData = self.buildRandomNumberList(1000)
        for point in inputData:
            histogram.addPoint(point)
        jsonHistogram = histogram.toJSON()
        self.assertAlmostEqual(jsonHistogram["average"], 0.0, places = 0)
        self.assertAlmostEqual(jsonHistogram["stdDev"], 1.0, places = 0)
        self.assertEqual(len(jsonHistogram["data"]), 16)
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 1000)
        pointsInHistogram = sum([x for x in viewvalues(jsonHistogram["data"])])

        # With high probability we must have chopped at least one point
        self.assertTrue(pointsInHistogram < 1000)
        self.assertAlmostEqual(pointsInHistogram / 1000.0, 0.68, places = 1)

        # Create a histogram without histogram data
        histogram = ContinuousSummaryHistogram('TestHisto', 'MyLabel', 'SomeoneElsesLabel',
                                               storeHistogram = False)
        inputData = self.buildRandomNumberList(1000)
        for point in inputData:
            histogram.addPoint(point)
        jsonHistogram = histogram.toJSON()
        self.assertAlmostEqual(jsonHistogram["average"], 0.0, places = 0)
        self.assertAlmostEqual(jsonHistogram["stdDev"], 1.0, places = 0)
        self.assertEqual(len(jsonHistogram["data"]), 0)
        self.assertEqual(jsonHistogram["internalData"]["nPoints"], 1000)

        return

if __name__ == "__main__":
    unittest.main()



'''
Created on Dec 16, 2016

'''
from __future__ import (division, print_function)

import unittest

from WMCore.Services.TagCollector.TagCollector import TagCollector


class TagCollectorTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        """
        # using the default production server
        self.tagCollecor = TagCollector()
        return

    def testTagCollecorMethods(self):
        """
        _testTagCollecorMethods_
        """
        releases = self.tagCollecor.releases()
        architectures = self.tagCollecor.architectures()
        realsese_by_arch = self.tagCollecor.releases_by_architecture()
        self.assertIn('CMSSW_7_1_10_patch2', releases)
        self.assertIn('slc6_amd64_gcc493', architectures)
        self.assertIn('slc7_amd64_gcc620', architectures)
        self.assertEqual(len(architectures), len(realsese_by_arch))
        self.assertEqual(sorted(self.tagCollecor.releases('slc6_amd64_gcc493')),
                         sorted(realsese_by_arch.get('slc6_amd64_gcc493')))

        return


if __name__ == '__main__':
    unittest.main()



"""
Oracle implementation of AddFile
"""
from WMCore.WMBS.MySQL.Files.Add import Add as AddFileMySQL

class Add(AddFileMySQL):
    """
    _Add_

    overwirtes MySQL Files.Add.sql to use oracle sequence instead of auto
    increments
    """
    sql = """INSERT INTO wmbs_file_details (id, lfn, filesize, events,
                                            first_event, merged)
             VALUES (wmbs_file_details_SEQ.nextval, :lfn, :filesize, :events,
                     :first_event, :merged)"""



"""
Unit tests for Unified/MSRuleCleaner.py module

"""
from __future__ import division, print_function

import json
# system modules
import os
import unittest

# WMCore modules
from WMCore.MicroService.MSRuleCleaner.MSRuleCleaner import MSRuleCleaner, MSRuleCleanerArchivalSkip
from WMCore.MicroService.MSRuleCleaner.MSRuleCleanerWflow import MSRuleCleanerWflow
from WMCore.Services.Rucio import Rucio


def getTestFile(partialPath):
    """
    Returns the absolute path for the test json file
    """
    normPath = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..'))
    return os.path.join(normPath, partialPath)


# class MSRuleCleanerTest(EmulatedUnitTestCase):
class MSRuleCleanerTest(unittest.TestCase):
    "Unit test for MSruleCleaner module"

    def setUp(self):
        "init test class"
        self.maxDiff = None
        self.msConfig = {"verbose": True,
                         "interval": 1 * 60,
                         "services": ['ruleCleaner'],
                         "rucioWmaAcct": 'wma_test',
                         "rucioAccount": 'wma_test',
                         'reqmgr2Url': 'https://cmsweb-testbed.cern.ch/reqmgr2',
                         'msOutputUrl': 'https://cmsweb-testbed.cern.ch/ms-output',
                         'reqmgrCacheUrl': 'https://cmsweb-testbed.cern.ch/couchdb/reqmgr_workload_cache',
                         'phedexUrl': 'https://cmsweb-testbed.cern.ch/phedex/datasvc/json/prod',
                         'dbsUrl': 'https://cmsweb-testbed.cern.ch/dbs/int/global/DBSReader',
                         'rucioUrl': 'http://cms-rucio-int.cern.ch',
                         'rucioAuthUrl': 'https://cms-rucio-auth-int.cern.ch',
                         "wmstatsUrl": "https://cmsweb-testbed.cern.ch/wmstatsserver",
                         "logDBUrl": "https://cmsweb-testbed.cern.ch/couchdb/wmstats_logdb",
                         'logDBReporter': 'reqmgr2ms_ruleCleaner',
                         'archiveDelayHours': 8,
                         'enableRealMode': False}

        self.creds = {"client_cert": os.getenv("X509_USER_CERT", "Unknown"),
                      "client_key": os.getenv("X509_USER_KEY", "Unknown")}
        self.rucioConfigDict = {"rucio_host": self.msConfig['rucioUrl'],
                                "auth_host": self.msConfig['rucioAuthUrl'],
                                "auth_type": "x509",
                                "account": self.msConfig['rucioAccount'],
                                "ca_cert": False,
                                "timeout": 30,
                                "request_retries": 3,
                                "creds": self.creds}

        self.reqStatus = ['announced', 'aborted-completed', 'rejected']
        self.msRuleCleaner = MSRuleCleaner(self.msConfig)
        self.msRuleCleaner.resetCounters()
        self.msRuleCleaner.rucio = Rucio.Rucio(self.msConfig['rucioAccount'],
                                               hostUrl=self.rucioConfigDict['rucio_host'],
                                               authUrl=self.rucioConfigDict['auth_host'],
                                               configDict=self.rucioConfigDict)

        self.taskChainFile = getTestFile('data/ReqMgr/requests/Static/TaskChainRequestDump.json')
        self.stepChainFile = getTestFile('data/ReqMgr/requests/Static/StepChainRequestDump.json')
        self.reqRecordsFile = getTestFile('data/ReqMgr/requests/Static/BatchRequestsDump.json')
        with open(self.reqRecordsFile) as fd:
            self.reqRecords = json.load(fd)
        with open(self.taskChainFile) as fd:
            self.taskChainReq = json.load(fd)
        with open(self.stepChainFile) as fd:
            self.stepChainReq = json.load(fd)
        super(MSRuleCleanerTest, self).setUp()

    def testPipelineAgentBlock(self):
        # Test plineAgentBlock:
        wflow = MSRuleCleanerWflow(self.taskChainReq)
        self.msRuleCleaner.plineAgentBlock.run(wflow)
        expectedWflow = {'CleanupStatus': {'plineAgentBlock': True},
                         'DataPileup': [],
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': u'/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'MCPileup': [],
                         'OutputDatasets': [
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': ['plineAgentBlock'],
                         'RequestName': u'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': u'announced',
                         'RequestTransition': [{u'DN': u'', u'Status': u'new', u'UpdateTime': 1606723304},
                                               {u'DN': u'', u'Status': u'assignment-approved',
                                                u'UpdateTime': 1606723305},
                                               {u'DN': u'', u'Status': u'assigned', u'UpdateTime': 1606723306},
                                               {u'DN': u'', u'Status': u'staging', u'UpdateTime': 1606723461},
                                               {u'DN': u'', u'Status': u'staged', u'UpdateTime': 1606723590},
                                               {u'DN': u'', u'Status': u'acquired', u'UpdateTime': 1606723968},
                                               {u'DN': u'', u'Status': u'running-open', u'UpdateTime': 1606724572},
                                               {u'DN': u'', u'Status': u'running-closed', u'UpdateTime': 1606724573},
                                               {u'DN': u'', u'Status': u'completed', u'UpdateTime': 1607018413},
                                               {u'DN': u'', u'Status': u'closed-out', u'UpdateTime': 1607347706},
                                               {u'DN': u'', u'Status': u'announced', u'UpdateTime': 1607359514}],
                         'RequestType': u'TaskChain',
                         'SubRequestType': u'',
                         'RulesToClean': {'plineAgentBlock': []},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False}
        self.assertDictEqual(wflow, expectedWflow)

    def testPipelineAgentCont(self):
        # Test plineAgentCont
        wflow = MSRuleCleanerWflow(self.taskChainReq)
        self.msRuleCleaner.plineAgentCont.run(wflow)
        expectedWflow = {'CleanupStatus': {'plineAgentCont': True},
                         'DataPileup': [],
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': u'/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'MCPileup': [],
                         'OutputDatasets': [
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': ['plineAgentCont'],
                         'RequestName': u'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': u'announced',
                         'RequestTransition': [{u'DN': u'', u'Status': u'new', u'UpdateTime': 1606723304},
                                               {u'DN': u'', u'Status': u'assignment-approved',
                                                u'UpdateTime': 1606723305},
                                               {u'DN': u'', u'Status': u'assigned', u'UpdateTime': 1606723306},
                                               {u'DN': u'', u'Status': u'staging', u'UpdateTime': 1606723461},
                                               {u'DN': u'', u'Status': u'staged', u'UpdateTime': 1606723590},
                                               {u'DN': u'', u'Status': u'acquired', u'UpdateTime': 1606723968},
                                               {u'DN': u'', u'Status': u'running-open', u'UpdateTime': 1606724572},
                                               {u'DN': u'', u'Status': u'running-closed', u'UpdateTime': 1606724573},
                                               {u'DN': u'', u'Status': u'completed', u'UpdateTime': 1607018413},
                                               {u'DN': u'', u'Status': u'closed-out', u'UpdateTime': 1607347706},
                                               {u'DN': u'', u'Status': u'announced', u'UpdateTime': 1607359514}],
                         'RequestType': u'TaskChain',
                         'SubRequestType': u'',
                         'RulesToClean': {'plineAgentCont': []},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False}
        self.assertDictEqual(wflow, expectedWflow)

    def testPipelineMSTrBlock(self):
        # Test plineAgentCont
        wflow = MSRuleCleanerWflow(self.taskChainReq)
        self.msRuleCleaner.plineMSTrBlock.run(wflow)
        expectedWflow = {'CleanupStatus': {'plineMSTrBlock': True},
                         'DataPileup': [],
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': u'/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'MCPileup': [],
                         'OutputDatasets': [
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': ['plineMSTrBlock'],
                         'RequestName': u'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': u'announced',
                         'RequestTransition': [{u'DN': u'',
                                                u'Status': u'new',
                                                u'UpdateTime': 1606723304},
                                               {u'DN': u'', u'Status': u'assignment-approved',
                                                u'UpdateTime': 1606723305},
                                               {u'DN': u'', u'Status': u'assigned', u'UpdateTime': 1606723306},
                                               {u'DN': u'', u'Status': u'staging', u'UpdateTime': 1606723461},
                                               {u'DN': u'', u'Status': u'staged', u'UpdateTime': 1606723590},
                                               {u'DN': u'', u'Status': u'acquired', u'UpdateTime': 1606723968},
                                               {u'DN': u'', u'Status': u'running-open', u'UpdateTime': 1606724572},
                                               {u'DN': u'', u'Status': u'running-closed', u'UpdateTime': 1606724573},
                                               {u'DN': u'', u'Status': u'completed', u'UpdateTime': 1607018413},
                                               {u'DN': u'', u'Status': u'closed-out', u'UpdateTime': 1607347706},
                                               {u'DN': u'', u'Status': u'announced', u'UpdateTime': 1607359514}],
                         'RequestType': u'TaskChain',
                         'SubRequestType': u'',
                         'RulesToClean': {'plineMSTrBlock': []},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False}
        self.assertDictEqual(wflow, expectedWflow)

    def testPipelineMSTrCont(self):
        # Test plineAgentCont
        wflow = MSRuleCleanerWflow(self.taskChainReq)
        self.msRuleCleaner.plineMSTrCont.run(wflow)
        expectedWflow = {'CleanupStatus': {'plineMSTrCont': True},
                         'DataPileup': [],
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': u'/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': False,
                         'IsClean': False,
                         'IsLogDBClean': False,
                         'MCPileup': [],
                         'OutputDatasets': [
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': ['plineMSTrCont'],
                         'RequestName': u'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': u'announced',
                         'RequestTransition': [{u'DN': u'',
                                                u'Status': u'new',
                                                u'UpdateTime': 1606723304},
                                               {u'DN': u'', u'Status': u'assignment-approved',
                                                u'UpdateTime': 1606723305},
                                               {u'DN': u'', u'Status': u'assigned', u'UpdateTime': 1606723306},
                                               {u'DN': u'', u'Status': u'staging', u'UpdateTime': 1606723461},
                                               {u'DN': u'', u'Status': u'staged', u'UpdateTime': 1606723590},
                                               {u'DN': u'', u'Status': u'acquired', u'UpdateTime': 1606723968},
                                               {u'DN': u'', u'Status': u'running-open', u'UpdateTime': 1606724572},
                                               {u'DN': u'', u'Status': u'running-closed', u'UpdateTime': 1606724573},
                                               {u'DN': u'', u'Status': u'completed', u'UpdateTime': 1607018413},
                                               {u'DN': u'', u'Status': u'closed-out', u'UpdateTime': 1607347706},
                                               {u'DN': u'', u'Status': u'announced', u'UpdateTime': 1607359514}],
                         'RequestType': u'TaskChain',
                         'SubRequestType': u'',
                         'RulesToClean': {'plineMSTrCont': []},
                         'TargetStatus': None,
                         'TransferDone': False,
                         'TransferTape': False}
        self.assertDictEqual(wflow, expectedWflow)

    def testPipelineArchive(self):
        # Test plineAgentCont
        wflow = MSRuleCleanerWflow(self.taskChainReq)

        # Try archival of a skipped workflow:
        with self.assertRaises(MSRuleCleanerArchivalSkip):
            self.msRuleCleaner.plineArchive.run(wflow)
        self.msRuleCleaner.plineAgentBlock.run(wflow)
        self.msRuleCleaner.plineAgentCont.run(wflow)

        # Try archival of a cleaned workflow:
        # NOTE: We should always expect an MSRuleCleanerArchivalSkip exception
        #       here because the 'enableRealRunMode' flag is set to False
        with self.assertRaises(MSRuleCleanerArchivalSkip):
            self.msRuleCleaner.plineArchive.run(wflow)
        expectedWflow = {'CleanupStatus': {'plineAgentBlock': True, 'plineAgentCont': True},
                         'DataPileup': [],
                         'ForceArchive': False,
                         'IncludeParents': False,
                         'InputDataset': u'/JetHT/Run2012C-v1/RAW',
                         'IsArchivalDelayExpired': True,
                         'IsClean': True,
                         'IsLogDBClean': True,
                         'MCPileup': [],
                         'OutputDatasets': [
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/RECO',
                             u'/JetHT/CMSSW_7_2_0-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/DQMIO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-SiStripCalZeroBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO',
                             u'/JetHT/CMSSW_7_2_0-TkAlMinBias-RECODreHLT_TaskChain_LumiMask_multiRun_HG2011_Val_Todor_v1-v11/ALCARECO'],
                         'ParentDataset': [],
                         'ParentageResolved': True,
                         'PlineMarkers': ['plineArchive',
                                          'plineAgentBlock',
                                          'plineAgentCont',
                                          'plineArchive'],
                         'RequestName': u'TaskChain_LumiMask_multiRun_HG2011_Val_201029_112735_5891',
                         'RequestStatus': u'announced',
                         'RequestTransition': [{u'DN': u'', u'Status': u'new', u'UpdateTime': 1606723304},
                                               {u'DN': u'', u'Status': u'assignment-approved',
                                                u'UpdateTime': 1606723305},
                                               {u'DN': u'', u'Status': u'assigned', u'UpdateTime': 1606723306},
                                               {u'DN': u'', u'Status': u'staging', u'UpdateTime': 1606723461},
                                               {u'DN': u'', u'Status': u'staged', u'UpdateTime': 1606723590},
                                               {u'DN': u'', u'Status': u'acquired', u'UpdateTime': 1606723968},
                                               {u'DN': u'', u'Status': u'running-open', u'UpdateTime': 1606724572},
                                               {u'DN': u'', u'Status': u'running-closed', u'UpdateTime': 1606724573},
                                               {u'DN': u'', u'Status': u'completed', u'UpdateTime': 1607018413},
                                               {u'DN': u'', u'Status': u'closed-out', u'UpdateTime': 1607347706},
                                               {u'DN': u'', u'Status': u'announced', u'UpdateTime': 1607359514}],
                         'RequestType': u'TaskChain',
                         'SubRequestType': u'',
                         'RulesToClean': {'plineAgentBlock': [], 'plineAgentCont': []},
                         'TargetStatus': 'normal-archived',
                         'TransferDone': False,
                         'TransferTape': False}
        self.assertDictEqual(wflow, expectedWflow)

        # Try archival of an uncleaned workflow
        wflow['CleanupStatus']['plineAgentBlock'] = False
        with self.assertRaises(MSRuleCleanerArchivalSkip):
            self.msRuleCleaner.plineArchive.run(wflow)

    def testRunning(self):
        result = self.msRuleCleaner._execute(self.reqRecords)
        self.assertEqual(result, (3, 2, 0, 0))

    def testCheckClean(self):
        # NOTE: All of the bellow checks are well visualized at:
        #       https://github.com/dmwm/WMCore/pull/10023#discussion_r520070925

        # 1. MaskList shorter than FlagList
        wflowFlags = {'CleanupStatus': {'plineAgentBlock': True, 'plineAgentCont': True, 'plineMStrCont': False},
                      'PlineMarkers': ['plineAgentBlock', 'plineAgentCont']}
        self.assertTrue(self.msRuleCleaner._checkClean(wflowFlags))

        wflowFlags = {'CleanupStatus': {'plineAgentBlock': False, 'plineAgentCont': True, 'plineMStrCont': True},
                      'PlineMarkers': ['plineAgentBlock', 'plineAgentCont']}
        self.assertFalse(self.msRuleCleaner._checkClean(wflowFlags))

        # 2. MaskList Empty
        wflowFlags = {'CleanupStatus': {'plineAgentBlock': True, 'plineAgentCont': True},
                      'PlineMarkers': []}
        self.assertFalse(self.msRuleCleaner._checkClean(wflowFlags))

        # 3. MaskList longer than FlagList
        wflowFlags = {'CleanupStatus': {'plineAgentBlock': True, 'plineAgentCont': True},
                      'PlineMarkers': ['plineAgentBlock', 'plineAgentCont', 'plineMStrCont', 'plineArchive']}
        self.assertTrue(self.msRuleCleaner._checkClean(wflowFlags))

        wflowFlags = {'CleanupStatus': {'plineAgentBlock': True, 'plineAgentCont': False},
                      'PlineMarkers': ['plineAgentBlock', 'plineAgentCont', 'plineMStrCont', 'plineArchive']}
        self.assertFalse(self.msRuleCleaner._checkClean(wflowFlags))

        # 4. FlagList Empty
        wflowFlags = {'CleanupStatus': {},
                      'PlineMarkers': ['plineAgentBlock', 'plineAgentCont']}
        self.assertFalse(self.msRuleCleaner._checkClean(wflowFlags))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#pylint: disable=E1101,C0103,R0902


import unittest

from Utils.PythonVersion import PY3

from WMCore.Configuration import ConfigSection
from WMCore.Configuration import Configuration
from WMCore.Configuration import ConfigurationEx
from WMCore.Configuration import loadConfigurationFile
from WMCore.Configuration import saveConfigurationFile

from WMQuality.TestInit import TestInit


class ConfigurationExTest(unittest.TestCase):
    """
    test case for Configuration object

    """
    def setUp(self):
        """set up"""
        self.testInit = TestInit(__file__)
        self.testDir  = self.testInit.generateWorkDir()
        self.functionSave = "%s/WMCore_Agent_Configuration_t_function.py" % self.testDir


    def tearDown(self):
        """clean up"""
        self.testInit.delWorkDir()


    def testCallableConfigParams(self):
        """ctor"""
        def f():
            return True

        config = Configuration()
        config.section_("SectionF")
        #creating field for the following test
        config.SectionF.aFunction = ''
        #Cannot set a function for plain Configuration objects
        #config.SectionF.__setattr__('aFunction', f)
        self.assertRaises(RuntimeError, config.SectionF.__setattr__, config.SectionF.aFunction, f)

        config = ConfigurationEx()
        config.section_("SectionF")
        #No failures with configurationEx
        config.SectionF.aFunction = f

        #However ConfigurationEx instances cannot be saved
        self.assertRaises(RuntimeError, saveConfigurationFile, config, self.functionSave)


class ConfigurationTest(unittest.TestCase):
    """
    test case for Configuration object

    """
    def setUp(self):
        """set up"""
        self.testInit = TestInit(__file__)
        self.testDir  = self.testInit.generateWorkDir()
        self.normalSave = "%s/WMCore_Agent_Configuration_t_normal.py" % self.testDir
        self.docSave = "%s/WMCore_Agent_Configuration_t_documented.py" % self.testDir
        self.commentSave = "%s/WMCore_Agent_Configuration_t_commented.py" % self.testDir
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        """clean up"""
        self.testInit.delWorkDir()


    def testA(self):
        """ctor"""
        try:
            dummyConfig = Configuration()
        except Exception as ex:
            msg = "Failed to instantiate Configuration\n"
            msg += str(ex)
            self.fail(msg)


    def testB(self):
        """add settings"""
        config = Configuration()
        config.section_("Section1")

        section1 = getattr(config, "Section1", None)
        self.assertTrue(section1 != None)

        config.section_("Section2")
        section2 = getattr(config, "Section2", None)
        self.assertTrue(section2 != None)

        self.assertRaises(AttributeError, getattr, config, "Section3")

        # basic types
        config.Section1.Parameter1 = True
        config.Section1.Parameter2 = "string"
        config.Section1.Parameter3 = 123
        config.Section1.Parameter4 = 123.456

        self.assertEqual(config.Section1.Parameter1, True)
        self.assertEqual(config.Section1.Parameter2, "string")
        self.assertEqual(config.Section1.Parameter3, 123)
        self.assertEqual(config.Section1.Parameter4, 123.456)

        # dictionary format:
        try:
            section1Dict = config.Section1.dictionary_()
        except Exception as ex:
            msg = "Error converting section to dictionary:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)

        self.assertTrue( "Parameter1" in section1Dict)
        self.assertTrue( "Parameter2" in section1Dict)
        self.assertTrue( "Parameter3" in section1Dict)
        self.assertTrue( "Parameter4" in section1Dict)

        self.assertEqual(section1Dict['Parameter1'],
                         config.Section1.Parameter1)
        self.assertEqual(section1Dict['Parameter2'],
                         config.Section1.Parameter2)
        self.assertEqual(section1Dict['Parameter3'],
                         config.Section1.Parameter3)
        self.assertEqual(section1Dict['Parameter4'],
                         config.Section1.Parameter4)

        # compound types
        config.Section2.List = ["string", 123, 123.456, False]
        config.Section2.Dictionary = { "string" : "string",
                                       "int" : 123,
                                       "float" : 123.456,
                                       "bool" : False}
        config.Section2.Tuple = ("string", 123, 123.456, False)


        self.assertEqual(config.Section2.List,
                         ["string", 123, 123.456, False])
        self.assertEqual(config.Section2.Tuple,
                         ("string", 123, 123.456, False))

        class DummyObject:
            pass
        # unsupported parameter type
        self.assertRaises(
            RuntimeError, setattr,
            config.Section2, "BadObject", DummyObject())
        # unsupported data type in compound type
        badList = [ DummyObject(), DummyObject()]
        self.assertRaises(
            RuntimeError, setattr,
            config.Section2, "BadList", badList)

        badDict = { "dict" : {}, "list": [DummyObject()], "tuple" : () }
        self.assertRaises(
            RuntimeError, setattr,
            config.Section2, "BadDict", badDict)

        goodDict = { "dict" : {}, "list": [], "tuple" : () }
        config.Section2.GoodDict = goodDict


    def testC(self):
        """add components"""
        config = Configuration()
        config.component_("Component1")
        config.component_("Component2")
        config.component_("Component3")

        comp1 = getattr(config, "Component1", None)
        self.assertTrue(comp1 != None)
        comp2 = getattr(config, "Component2", None)
        self.assertTrue(comp2 != None)


    def testD(self):
        """test documentation"""
        config = Configuration()
        config.section_("Section1")
        config.Section1.Parameter1 = True
        config.Section1.Parameter2 = "string"
        config.Section1.Parameter3 = 123
        config.Section1.Parameter4 = 123.456
        config.Section1.Parameter5 = {"test1" : "test2", "test3" : 123}

        config.Section1.document_("""This is Section1""")
        config.Section1.document_("""This is Section1.Parameter1""",
                                  "Parameter1")
        config.Section1.document_("""This is Section1.Parameter2""",
                                  "Parameter2")
        config.Section1.document_("""This is Section1.Parameter3\n with multiline comments""",
                                  "Parameter3")

        try:
            config.Section1.documentedString_()
        except Exception as ex:
            msg = "Error calling ConfigSection.documentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)
        try:
            config.Section1.commentedString_()
        except Exception as ex:
            msg = "Error calling ConfigSection.commentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)

        try:
            config.documentedString_()
        except Exception as ex:
            msg = "Error calling Configuration.documentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)
        try:
            config.commentedString_()
        except Exception as ex:
            msg = "Error calling Configuration.commentedString_:\n"
            msg += "%s\n" % str(ex)
            self.fail(msg)


    def testE(self):
        """test save/load """
        testValues = [
            "string", 123, 123.456,
            ["list", 789, 10.1 ],
            { "dict1" : "value", "dict2" : 10.0 }
            ]
        config = Configuration()
        for x in range(0, 5):
            config.section_("Section%s" % x)
            config.component_("Component%s" % x)
            sect = getattr(config, "Section%s" % x)
            comp = getattr(config, "Component%s" % x)
            sect.document_("This is Section%s" % x)
            comp.document_("This is Component%s" % x)

            for i in range(0, 5):
                setattr(comp, "Parameter%s" % i, testValues[i])
                setattr(sect, "Parameter%s" % i, testValues[i])
                comp.document_("This is Parameter%s" % i,
                               "Parameter%s" %i)
                sect.document_("This is Parameter%s" %i,
                               "Parameter%s" %i)

        dummyStringSave = str(config)
        dummyDocumentSave = config.documentedString_()
        dummyCommentSave = config.commentedString_()

        saveConfigurationFile(config, self.normalSave)
        saveConfigurationFile(config, self.docSave, document = True)
        saveConfigurationFile(config, self.commentSave, comment = True)

        dummyPlainConfig = loadConfigurationFile(self.normalSave)

        dummyDocConfig = loadConfigurationFile(self.docSave)

        dummyCommentConfig = loadConfigurationFile(self.commentSave)

        #print commentConfig.commentedString_()
        #print docConfig.documentedString_()
        #print docConfig.commentedString_()


    def testF(self):
        """
        Test internal functions pythonise_, listSections_
        """
        config = ConfigSection("config")

        config.section_("SectionA")
        config.section_("SectionB")
        config.SectionA.section_("Section1")
        config.SectionA.section_("Section2")
        config.SectionA.Section1.x   = 100
        config.SectionA.Section1.y   = 100

        pythonise = config.pythonise_()

        assert "config.section_('SectionA')"      in pythonise, "Pythonise failed: Could not find SectionA"
        assert "config.SectionA.Section1.x = 100" in pythonise, "Pythonise failed: Could not find x"

        pythonise = config.SectionA.pythonise_()

        assert "SectionA.section_('Section1')" in pythonise, "Pythonise failed: Could not find Section1"
        assert "SectionA.Section1.x = 100"     in pythonise, "Pythonise failed: Could not find x"

        self.assertItemsEqual(config.listSections_(), ['SectionB', 'SectionA'])
        self.assertItemsEqual(config.SectionA.listSections_(), ['Section2', 'Section1'])


    def testG_testStaticReferenceToConfigurationInstance(self):
        """
        test Configuration.getInstance() which returns reference
        to the Configuration object instance.

        """
        config = Configuration()
        instance = Configuration.getInstance()
        self.assertFalse(hasattr(instance, "testsection"))
        config.section_("testsection")
        self.assertTrue(hasattr(instance, "testsection"))
        config.testsection.var = 10
        self.assertEqual(instance.testsection.var, 10)


    def testH_ConfigSectionDictionariseInternalChildren(self):
        """
        The test checks if any item of the dictionary_whole_tree_()
        result is not unexpanded instance of ConfigSection.

        """
        config = ConfigSection("config")
        config.value1 = "MyValue1"
        config.section_("Task1")
        config.Task1.value2 = "MyValue2"
        config.Task1.section_("subSection")
        config.Task1.subSection.value3 = "MyValue3"
        d = config.dictionary_whole_tree_()
        for values in d.values():
            self.assertFalse(isinstance(values, ConfigSection))
        self.assertEqual(d["Task1"]["subSection"]["value3"], "MyValue3")


    def testI_testGetInternalName(self):
        """
        Test we properly retrieve the internal name of the configuration object

        """
        config = ConfigSection("config")
        name = config.getInternalName()
        self.assertEqual(name, "config")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
_Report_t_

Unit tests for the Report class.
"""

from future.utils import viewvalues
from Utils.Utilities import encodeUnicodeToBytes, decodeBytesToUnicode

import os
import time
import unittest

from Utils import FileTools
from Utils.PythonVersion import PY3

from WMCore.Configuration import ConfigSection
from WMCore.FwkJobReport.Report import Report
from WMCore.WMBase import getTestBase
from WMQuality.TestInitCouchApp import TestInitCouchApp


class ReportTest(unittest.TestCase):
    """
    _ReportTest_

    Unit tests for the Report class.
    """

    def setUp(self):
        """
        _setUp_

        Figure out the location of the XML report produced by CMSSW.
        """
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection(destroyAllDatabase=True)
        self.testInit.setupCouch("report_t/fwjrs", "FWJRDump")

        testData = os.path.join(getTestBase(), "WMCore_t/FwkJobReport_t")
        self.xmlPath = os.path.join(testData, "CMSSWProcessingReport.xml")
        self.badxmlPath = os.path.join(testData, "CMSSWFailReport2.xml")
        self.skippedFilesxmlPath = os.path.join(testData, "CMSSWSkippedNonExistentFile.xml")
        self.skippedAllFilesxmlPath = os.path.join(testData, "CMSSWSkippedAll.xml")
        self.fallbackXmlPath = os.path.join(testData, "CMSSWInputFallback.xml")
        self.twoFileFallbackXmlPath = os.path.join(testData, "CMSSWTwoFileRemote.xml")
        self.pileupXmlPath = os.path.join(testData, "CMSSWPileup.xml")
        self.withEventsXmlPath = os.path.join(testData, "CMSSWWithEventCounts.xml")
        self.noLocationReport = os.path.join(testData, "Report.0.pkl")

        self.testDir = self.testInit.generateWorkDir()

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

        return

    def tearDown(self):
        """
        _tearDown_

        Cleanup the databases.
        """
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        self.testInit.delWorkDir()
        return

    def verifyInputData(self, report):
        """
        _verifyInputData_

        Verify that the input file in the Report class matches the input file in
        the XML generated by CMSSW.
        """
        inputFiles = report.getInputFilesFromStep("cmsRun1")

        assert len(inputFiles) == 1, \
            "Error: Wrong number of input files."
        assert inputFiles[0][
                   "lfn"] == "/store/data/BeamCommissioning09/MinimumBias/RAW/v1/000/122/023/142F3F42-C5D6-DE11-945D-000423D94494.root", \
            "Error: Wrong LFN on input file."
        assert inputFiles[0][
                   "pfn"] == "dcap://cmsdca.fnal.gov:24137/pnfs/fnal.gov/usr/cms/WAX/11/store/data/BeamCommissioning09/MinimumBias/RAW/v1/000/122/023/142F3F42-C5D6-DE11-945D-000423D94494.root", \
            "Error: Wrong PFN on input file."

        inputRun = list(inputFiles[0]["runs"])
        assert len(inputRun) == 1, \
            "Error: Wrong number of runs in input."
        assert inputRun[0].run == 122023, \
            "Error: Wrong run number on input file."
        assert len(inputRun[0].lumis) == 1, \
            "Error: Wrong number of lumis in input file."
        assert 215 in inputRun[0].lumis, \
            "Error: Input file is missing lumis."

        assert inputFiles[0]["events"] == 2, \
            "Error: Wrong number of events in input file."
        assert inputFiles[0]["size"] == 0, \
            "Error: Wrong size in input file."

        assert inputFiles[0][
                   "catalog"] == "trivialcatalog_file:/uscmst1/prod/sw/cms/SITECONF/T1_US_FNAL/PhEDEx/storage.xml?protocol=dcap", \
            "Error: Catalog on input file is wrong."
        assert inputFiles[0]["guid"] == "142F3F42-C5D6-DE11-945D-000423D94494", \
            "Error: GUID of input file is wrong."

        return

    def verifyRecoOutput(self, report, hasEventCounts=False):
        """
        _verifyRecoOutput_

        Verify that all the metadata in the RECO output module is correct.
        """
        outputFiles = report.getFilesFromOutputModule("cmsRun1", "outputRECORECO")

        assert len(outputFiles) == 1, \
            "Error: Wrong number of output files."
        assert outputFiles[0][
                   "lfn"] == "/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/RECO/rereco_GR09_R_34X_V5_All_v1/0000/outputRECORECO.root", \
            "Error: Wrong LFN on output file: %s" % outputFiles[0]["lfn"]
        assert outputFiles[0]["pfn"] == "outputRECORECO.root", \
            "Error: Wrong PFN on output file."

        outputRun = list(outputFiles[0]["runs"])
        assert len(outputRun) == 1, \
            "Error: Wrong number of runs in output."
        assert outputRun[0].run == 122023, \
            "Error: Wrong run number on output file."
        assert len(outputRun[0].lumis) == 1, \
            "Error: Wrong number of lumis in output file."
        assert 215 in outputRun[0].lumis, \
            "Error: Output file is missing lumis."

        self.assertIsInstance(outputRun[0].eventsPerLumi, dict)
        if hasEventCounts:
            self.assertEqual(outputRun[0].eventsPerLumi[215], 2)
        else:
            self.assertIsNone(outputRun[0].eventsPerLumi[215])

        assert outputFiles[0]["events"] == 2, \
            "Error: Wrong number of events in output file."
        assert outputFiles[0]["size"] == 0, \
            "Error: Wrong size in output file."

        assert len(outputFiles[0]["input"]) == 1, \
            "Error: Wrong number of input files."
        assert outputFiles[0]["input"][
                   0] == "/store/data/BeamCommissioning09/MinimumBias/RAW/v1/000/122/023/142F3F42-C5D6-DE11-945D-000423D94494.root", \
            "Error: LFN of input file is wrong."

        assert len(outputFiles[0]["checksums"]) == 0, \
            "Error: There should be no checksums in output file."
        assert outputFiles[0]["catalog"] == "", \
            "Error: Catalog on output file is wrong."
        assert outputFiles[0]["guid"] == "7E3359C8-222E-DF11-B2B0-001731230E47", \
            "Error: GUID of output file is wrong: %s" % outputFiles[0]["guid"]
        assert outputFiles[0]["module_label"] == "outputRECORECO", \
            "Error: Module label of output file is wrong."
        assert outputFiles[0]["branch_hash"] == "cf37adeb60b427f4ccd0e21b5771146b", \
            "Error: Branch has on output file is wrong."

        return

    def verifyAlcaOutput(self, report, hasEventCounts=False):
        """
        _verifyAlcaOutput_

        Verify that all of the meta data in the ALCARECO output module is
        correct.
        """
        outputFiles = report.getFilesFromOutputModule("cmsRun1", "outputALCARECORECO")
        assert len(outputFiles) == 1, \
            "Error: Wrong number of output files."
        assert outputFiles[0][
                   "lfn"] == "/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/ALCARECO/rereco_GR09_R_34X_V5_All_v1/0000/B8F849C9-222E-DF11-B2B0-001731230E47.root", \
            "Error: Wrong LFN on output file: %s" % outputFiles[0]["lfn"]
        assert outputFiles[0]["pfn"] == "outputALCARECORECO.root", \
            "Error: Wrong PFN on output file."

        outputRun = list(outputFiles[0]["runs"])
        assert len(outputRun) == 1, \
            "Error: Wrong number of runs in output."
        assert outputRun[0].run == 122023, \
            "Error: Wrong run number on output file."
        assert len(outputRun[0].lumis) == 1, \
            "Error: Wrong number of lumis in output file."
        assert 215 in outputRun[0].lumis, \
            "Error: Output file is missing lumis."

        self.assertIsInstance(outputRun[0].eventsPerLumi, dict)
        if hasEventCounts:
            self.assertIsNone(outputRun[0].eventsPerLumi[215])
        else:
            self.assertIsNone(outputRun[0].eventsPerLumi[215])

        assert outputFiles[0]["events"] == 2, \
            "Error: Wrong number of events in output file."
        assert outputFiles[0]["size"] == 0, \
            "Error: Wrong size in output file."

        assert len(outputFiles[0]["input"]) == 1, \
            "Error: Wrong number of input files."
        assert outputFiles[0]["input"][
                   0] == "/store/data/BeamCommissioning09/MinimumBias/RAW/v1/000/122/023/142F3F42-C5D6-DE11-945D-000423D94494.root", \
            "Error: LFN of input file is wrong."

        assert len(outputFiles[0]["checksums"]) == 0, \
            "Error: There should be no checksums in output file."
        assert outputFiles[0]["catalog"] == "", \
            "Error: Catalog on output file is wrong."
        assert outputFiles[0]["guid"] == "B8F849C9-222E-DF11-B2B0-001731230E47", \
            "Error: GUID of output file is wrong: %s" % outputFiles[0]["guid"]
        assert outputFiles[0]["module_label"] == "outputALCARECORECO", \
            "Error: Module label of output file is wrong."
        assert outputFiles[0]["branch_hash"] == "cf37adeb60b427f4ccd0e21b5771146b", \
            "Error: Branch has on output file is wrong."

        return

    def testXMLParsing(self):
        """
        _testParsing_

        Verify that the parsing of a CMSSW XML report works correctly.
        """
        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        self.verifyInputData(myReport)
        self.verifyRecoOutput(myReport)
        self.verifyAlcaOutput(myReport)

        return

    def testWithEventsXMLParsing(self):
        """
        _testParsing_

        Verify that the parsing of a CMSSW XML report works correctly.
        """
        myReport = Report("cmsRun1")
        myReport.parse(self.withEventsXmlPath)

        self.verifyInputData(myReport)
        self.verifyRecoOutput(myReport, hasEventCounts=True)
        self.verifyAlcaOutput(myReport, hasEventCounts=True)

        return

    def testBadXMLParsing(self):
        """
        _testBadXMLParsing_

        Verify that the parsing of a CMSSW XML report works correctly even if
        the XML is malformed.

        This should raise a FwkJobReportException, which in CMSSW will be caught
        """
        myReport = Report("cmsRun1")
        from WMCore.FwkJobReport.Report import FwkJobReportException
        self.assertRaises(FwkJobReportException, myReport.parse, self.badxmlPath)
        return

    def testErrorReporting(self):
        """
        _testErrorReporting_

        Verify that errors are correctly transfered from the XML report to the
        python report.
        """
        cmsException = "cms::Exception caught in cmsRun\n"
        cmsException += "---- EventProcessorFailure BEGIN\n"
        cmsException += "EventProcessingStopped\n"
        cmsException += "---- ScheduleExecutionFailure BEGIN\n"
        cmsException += "ProcessingStopped\n"
        cmsException += "---- NoRecord BEGIN\n"
        cmsException += 'No "CastorDbRecord" record found in the EventSetup.\n'
        cmsException += " Please add an ESSource or ESProducer that delivers such a record.\n"
        cmsException += "cms::Exception going through module CastorRawToDigi/castorDigis run: 121849 lumi: 1 event: 23\n"
        cmsException += "---- NoRecord END\n"
        cmsException += "Exception going through path raw2digi_step\n"
        cmsException += "---- ScheduleExecutionFailure END\n"
        cmsException += "an exception occurred during current event processing\n"
        cmsException += "cms::Exception caught in EventProcessor and rethrown\n"
        cmsException += "---- EventProcessorFailure END"

        xmlPath = os.path.join(getTestBase(),
                               "WMCore_t/FwkJobReport_t/CMSSWFailReport.xml")

        myReport = Report("cmsRun1")
        myReport.parse(xmlPath)

        assert hasattr(myReport.data.cmsRun1, "errors"), \
            "Error: Error section missing."
        assert getattr(myReport.data.cmsRun1.errors, "errorCount") == 1, \
            "Error: Error count is wrong."
        assert hasattr(myReport.data.cmsRun1.errors, "error0"), \
            "Error: Error0 section is missing."
        assert myReport.data.cmsRun1.errors.error0.type == "CMSException", \
            "Error: Wrong error type."
        assert myReport.data.cmsRun1.errors.error0.exitCode == 8001, \
            "Error: Wrong exit code."
        assert myReport.data.cmsRun1.errors.error0.details == cmsException, \
            "Error: Error details are wrong:\n|%s|\n|%s|" % (myReport.data.cmsRun1.errors.error0.details,
                                                             cmsException)

        # Test getStepErrors
        self.assertEqual(myReport.getStepErrors("cmsRun1")['error0'].type, "CMSException")

        return

    def testMultipleInputs(self):
        """
        _testMultipleInputs_

        Verify that parsing XML reports with multiple inputs works correctly.
        """
        xmlPath = os.path.join(getTestBase(),
                               "WMCore_t/FwkJobReport_t/CMSSWMultipleInput.xml")
        myReport = Report("cmsRun1")
        myReport.parse(xmlPath)

        assert hasattr(myReport.data.cmsRun1.input, "source"), \
            "Error: Report missing input source."

        inputFiles = myReport.getInputFilesFromStep("cmsRun1")

        assert len(inputFiles) == 2, \
            "Error: Wrong number of input files."

        for inputFile in inputFiles:
            assert inputFile["input_type"] == "primaryFiles", \
                "Error: Wrong input type."
            assert inputFile["module_label"] == "source", \
                "Error: Module label is wrong"
            assert inputFile[
                       "catalog"] == "trivialcatalog_file:/uscmst1/prod/sw/cms/SITECONF/T1_US_FNAL/PhEDEx/storage.xml?protocol=dcap", \
                "Error: Catalog is wrong."
            assert inputFile["events"] == 2, \
                "Error: Wrong number of events."
            assert inputFile["input_source_class"] == "PoolSource", \
                "Error: Wrong input source class."

            if inputFile["guid"] == "F0875ECD-3347-DF11-9FE0-003048678A80":
                assert inputFile[
                           "lfn"] == "/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/RECO/rereco_GR10_P_V4_All_v1/0000/F0875ECD-3347-DF11-9FE0-003048678A80.root", \
                    "Error: Input LFN is wrong."
                assert inputFile[
                           "pfn"] == "dcap://cmsdca3.fnal.gov:24142/pnfs/fnal.gov/usr/cms/WAX/11/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/RECO/rereco_GR10_P_V4_All_v1/0000/F0875ECD-3347-DF11-9FE0-003048678A80.root", \
                    "Error: Input PFN is wrong."
                assert len(inputFile["runs"]) == 1, \
                    "Error: Wrong number of runs."
                assert list(inputFile["runs"])[0].run == 124216, \
                    "Error: Wrong run number."
                assert 1 in list(inputFile["runs"])[0], \
                    "Error: Wrong lumi sections in input file."
            else:
                assert inputFile["guid"] == "626D74CE-3347-DF11-9363-0030486790C0", \
                    "Error: Wrong guid."
                assert inputFile[
                           "lfn"] == "/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/RECO/rereco_GR10_P_V4_All_v1/0000/626D74CE-3347-DF11-9363-0030486790C0.root", \
                    "Error: Input LFN is wrong."
                assert inputFile[
                           "pfn"] == "dcap://cmsdca3.fnal.gov:24142/pnfs/fnal.gov/usr/cms/WAX/11/store/backfill/2/unmerged/WMAgentCommissioining10/MinimumBias/RECO/rereco_GR10_P_V4_All_v1/0000/626D74CE-3347-DF11-9363-0030486790C0.root", \
                    "Error: Input PFN is wrong."
                assert len(inputFile["runs"]) == 1, \
                    "Error: Wrong number of runs."
                assert list(inputFile["runs"])[0].run == 124216, \
                    "Error: Wrong run number."
                assert 2 in list(inputFile["runs"])[0], \
                    "Error: Wrong lumi sections in input file."

        return

    def testJSONEncoding(self):
        """
        _testJSONEncoding_

        Verify that turning the FWJR into a JSON object works correctly.
        """
        xmlPath = os.path.join(getTestBase(),
                               "WMCore_t/FwkJobReport_t/CMSSWProcessingReport.xml")
        myReport = Report("cmsRun1")
        myReport.parse(xmlPath)

        jsonReport = myReport.__to_json__(None)

        assert "task" in jsonReport, \
            "Error: Task name missing from report."

        assert len(jsonReport["steps"]) == 1, \
            "Error: Wrong number of steps in report."
        assert "cmsRun1" in jsonReport["steps"], \
            "Error: Step missing from json report."

        cmsRunStep = jsonReport["steps"]["cmsRun1"]

        jsonReportSections = ["status", "errors", "logs", "parameters", "site",
                              "analysis", "cleanup", "input", "output", "start"]
        for jsonReportSection in jsonReportSections:
            assert jsonReportSection in cmsRunStep, \
                "Error: missing section: %s" % jsonReportSection

        return

    def testTimeSetting(self):
        """
        _testTimeSetting_

        Can we set the times correctly?
        """
        stepName = "cmsRun1"
        timeDiff = 0.01
        myReport = Report(stepName)
        localTime = time.time()
        myReport.setStepStartTime(stepName)
        myReport.setStepStopTime(stepName)
        repTime = myReport.getTimes(stepName)

        self.assertTrue(repTime["startTime"] - localTime < timeDiff)
        self.assertTrue(repTime["stopTime"] - localTime < timeDiff)

        myReport = Report("cmsRun1")
        myReport.addStep("cmsRun2")
        myReport.addStep("cmsRun3")

        step = myReport.retrieveStep("cmsRun1")
        step.startTime = 1
        step.stopTime = 8
        step = myReport.retrieveStep("cmsRun2")
        step.startTime = 2
        step.stopTime = 9
        step = myReport.retrieveStep("cmsRun3")
        step.startTime = 3
        step.stopTime = 10

        self.assertEqual(myReport.getFirstStartLastStop()['stopTime'], 10)
        self.assertEqual(myReport.getFirstStartLastStop()['startTime'], 1)

        return

    def testTaskJobID(self):
        """
        _testTaskJobID_

        Test the basic task and jobID functions
        """

        report = Report('fake')
        self.assertEqual(report.getTaskName(), None)
        self.assertEqual(report.getJobID(), None)
        report.setTaskName('silly')
        report.setJobID(100)
        self.assertEqual(report.getTaskName(), 'silly')
        self.assertEqual(report.getJobID(), 100)

        return

    def test_PerformanceReport(self):
        """
        _PerformanceReport_

        Test the performance report part of the job report
        """

        report = Report("cmsRun1")
        report.setStepVSize(stepName="cmsRun1", minimum=100, maximum=800, average=244)
        report.setStepRSS(stepName="cmsRun1", minimum=100, maximum=800, average=244)
        report.setStepPCPU(stepName="cmsRun1", minimum=100, maximum=800, average=244)
        report.setStepPMEM(stepName="cmsRun1", minimum=100, maximum=800, average=244)

        perf = report.retrieveStep("cmsRun1").performance
        for section in viewvalues(perf.dictionary_()):
            d = section.dictionary_()
            self.assertEqual(d['min'], 100)
            self.assertEqual(d['max'], 800)
            self.assertEqual(d['average'], 244)
        return

    def testPerformanceSummary(self):
        """
        _testPerformanceSummary_

        Test whether or not we can pull performance information
        out of a Timing/SimpleMemoryCheck jobReport
        """

        xmlPath = os.path.join(getTestBase(),
                               "WMCore_t/FwkJobReport_t/PerformanceReport.xml")

        myReport = Report("cmsRun1")
        myReport.parse(xmlPath)

        # Do a brief check of the three sections
        perf = myReport.data.cmsRun1.performance

        self.assertEqual(perf.memory.PeakValueRss, '492.293')
        self.assertEqual(perf.cpu.TotalJobCPU, '9.16361')
        self.assertEqual(perf.storage.writeTotalMB, 5.22226)
        self.assertAlmostEqual(perf.storage.writeTotalSecs, 0, places=0)  # actual value is 0.06
        self.assertEqual(perf.storage.readPercentageOps, 0.98585512216030857)

        return

    def testPerformanceJSON(self):
        """
        _testPerformanceJSON_

        Verify that the performance section of the report is correctly converted
        to JSON.
        """
        xmlPath = os.path.join(getTestBase(),
                               "WMCore_t/FwkJobReport_t/PerformanceReport.xml")

        myReport = Report("cmsRun1")
        myReport.parse(xmlPath)

        perfSection = myReport.__to_json__(thunker=None)["steps"]["cmsRun1"]["performance"]

        self.assertTrue("storage" in perfSection,
                        "Error: Storage section is missing.")
        self.assertTrue("memory" in perfSection,
                        "Error: Memory section is missing.")
        self.assertTrue("cpu" in perfSection,
                        "Error: CPU section is missing.")

        self.assertEqual(perfSection["cpu"]["AvgEventCPU"], "0.626105",
                         "Error: AvgEventCPU is wrong.")
        self.assertEqual(perfSection["cpu"]["TotalJobTime"], "23.5703",
                         "Error: TotalJobTime is wrong.")
        self.assertEqual(perfSection["storage"]["readTotalMB"], 39.6166,
                         "Error: readTotalMB is wrong.")
        self.assertEqual(perfSection["storage"]["readMaxMSec"], 320.653,
                         "Error: readMaxMSec is wrong")
        self.assertEqual(perfSection["memory"]["PeakValueRss"], "492.293",
                         "Error: PeakValueRss is wrong.")
        self.assertEqual(perfSection["memory"]["PeakValueVsize"], "643.281",
                         "Error: PeakValueVsize is wrong.")
        return

    def testExitCode(self):
        """
        _testExitCode_

        Test and see if we can get an exit code out of a report

        Note: Errors without a return code return 99999
        getStepExitCode: returns the first valid and non-zero exit code
        getExitCode: uses the method above to get an exit code
        getStepExitCodes: returns a set of all exit codes within the step
        """

        report = Report("cmsRun1")
        self.assertEqual(report.getExitCode(), 0)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 0)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {})
        self.assertItemsEqual(report.getStepErrors(stepName="cmsRun1"), {})

        report.addError(stepName="cmsRun1", exitCode=None, errorType="test", errorDetails="test")
        # None is not a valid exitCode, but it will get mapped to 99999
        self.assertEqual(report.getExitCode(), 99999)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 99999)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 1)

        report.addError(stepName="cmsRun1", exitCode=102, errorType="test", errorDetails="test")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 2)

        report.addError(stepName="cmsRun1", exitCode=103, errorType="test", errorDetails="test")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 3)

        # now try to record the same exit code once again
        report.addError(stepName="cmsRun1", exitCode=104, errorType="test", errorDetails="test")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 4)

        # and once again, but different type and details (which does not matter)
        report.addError(stepName="cmsRun1", exitCode=105, errorType="testEE", errorDetails="testAA")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 5)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=106, errorType="test", errorDetails="1 тℯṧт")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 6)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=107, errorType="test", errorDetails="2 тℯṧт \x95")
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106, 107})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 7)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=108, errorType="test", errorDetails=encodeUnicodeToBytes("3 тℯṧт"))
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106, 107, 108})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 8)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=109, errorType="test", errorDetails=decodeBytesToUnicode("4 тℯṧт"))
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106, 107, 108, 109})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 9)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=110, errorType="test", errorDetails={"нεʟʟ◎": 3.14159})
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106, 107, 108, 109, 110})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 10)

        # and once again, but different type and details - testing unicode handling
        report.addError(stepName="cmsRun1", exitCode=111, errorType="test", errorDetails={"нεʟʟ◎ \x95": "ẘøґℓ∂ \x95"})
        self.assertEqual(report.getExitCode(), 102)
        self.assertEqual(report.getStepExitCode(stepName="cmsRun1"), 102)
        self.assertItemsEqual(report.getStepExitCodes(stepName="cmsRun1"), {99999, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111})
        self.assertEqual(report.getStepErrors(stepName="cmsRun1")['errorCount'], 11)

    def testProperties(self):
        """
        _testProperties_

        Test data fields for the properties information for DBS
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        name = "ThisIsASillyString"

        myReport.setValidStatus(name)
        myReport.setGlobalTag(name)
        myReport.setAcquisitionProcessing(acquisitionEra='NULL', processingVer=name)
        myReport.setInputDataset(inputPath='/lame/path')

        for f in myReport.getAllFilesFromStep("cmsRun1"):
            self.assertEqual(f['globalTag'], name)
            self.assertEqual(f['validStatus'], name)
            self.assertEqual(f['processingVer'], name)
            self.assertEqual(f['acquisitionEra'], 'NULL')
            self.assertEqual(f['inputPath'], '/lame/path')

        return

    def testOutputFiles(self):
        """
        _testOutputFiles_

        Test some basic manipulation of output files
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        files = myReport.getAllFilesFromStep(step="cmsRun1")

        f1 = files[0]
        f2 = files[1]

        self.assertEqual(f1['outputModule'], 'outputRECORECO')
        self.assertEqual(f1['pfn'], 'outputRECORECO.root')

        self.assertEqual(f2['outputModule'], 'outputALCARECORECO')
        self.assertEqual(f2['pfn'], 'outputALCARECORECO.root')

        for f in files:
            self.assertEqual(f['events'], 2)
            self.assertEqual(f['configURL'], None)
            self.assertEqual(f['merged'], False)
            self.assertEqual(f['validStatus'], None)
            self.assertEqual(f['first_event'], 0)

        return

    def testGetAdlerChecksum(self):
        """
        _testGetAdlerChecksum_

        Test the function that sees if all files
        have an adler checksum.

        For some reason, our default XML report doesn't have checksums
        Therefore it should fail.
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        myReport.checkForAdlerChecksum(stepName="cmsRun1")

        self.assertFalse(myReport.stepSuccessful(stepName="cmsRun1"))
        self.assertEqual(myReport.getExitCode(), 60451)

        # Now see what happens if the adler32 is set to None
        myReport2 = Report("cmsRun1")
        myReport2.parse(self.xmlPath)
        fRefs = myReport2.getAllFileRefsFromStep(step="cmsRun1")
        for fRef in fRefs:
            fRef.checksums = {'adler32': None}
        myReport2.checkForAdlerChecksum(stepName="cmsRun1")
        self.assertFalse(myReport2.stepSuccessful(stepName="cmsRun1"))
        self.assertEqual(myReport2.getExitCode(), 60451)

        myReport3 = Report("cmsRun1")
        myReport3.parse(self.xmlPath)
        fRefs = myReport3.getAllFileRefsFromStep(step="cmsRun1")
        for fRef in fRefs:
            fRef.checksums = {'adler32': 100}

        myReport3.checkForAdlerChecksum(stepName="cmsRun1")
        self.assertTrue(myReport3.getExitCode() != 60451)

        return

    def testCheckLumiInformation(self):
        """
        _testCheckLumiInformation_

        Test the function that checks if all files
        have run lumi information
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        myReport.checkForRunLumiInformation(stepName="cmsRun1")

        self.assertNotEqual(myReport.getExitCode(), 70452)

        # Remove the lumi information on purpose
        myReport2 = Report("cmsRun1")
        myReport2.parse(self.xmlPath)
        fRefs = myReport2.getAllFileRefsFromStep(step="cmsRun1")
        for fRef in fRefs:
            fRef.runs = ConfigSection()
        myReport2.checkForRunLumiInformation(stepName="cmsRun1")
        self.assertFalse(myReport2.stepSuccessful(stepName="cmsRun1"))
        self.assertEqual(myReport2.getExitCode(), 70452)

        return

    def testTaskSuccessful(self):
        """
        _testTaskSuccessful_

        Test whether or not the report marks the task successful
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        # First, the report should fail
        self.assertFalse(myReport.taskSuccessful())

        # Second, if we ignore cmsRun, the task
        # should succeed
        self.assertTrue(myReport.taskSuccessful(ignoreString='cmsRun'))
        return

    def testStripReport(self):
        """
        _testStripReport_

        Test whether or not we can strip input file information
        from a FWJR and create a smaller object.
        """

        myReport = Report("cmsRun1")
        myReport.parse(self.xmlPath)

        path1 = os.path.join(self.testDir, 'testReport1.pkl')
        path2 = os.path.join(self.testDir, 'testReport2.pkl')

        myReport.save(path1)
        info = FileTools.getFileInfo(filename=path1)
        sizeBefore = info['Size']

        inputFiles = myReport.getAllInputFiles()
        self.assertEqual(len(inputFiles), 1)
        myReport.stripInputFiles()
        self.assertEqual(len(myReport.getAllInputFiles()), 0)

        myReport.save(path2)
        info = FileTools.getFileInfo(filename=path2)
        sizeAfter = info['Size']

        self.assertGreater(sizeBefore, sizeAfter)

        return

    def testDuplicatStep(self):
        """
        _testDuplicateStep_

        If the same step is added twice, it should act
        as a replacement, and raise an appropriate message
        """

        baseReport = Report("cmsRun1")
        baseReport.parse(self.xmlPath)

        modReport = Report("cmsRun1")
        modReport.parse(self.xmlPath)
        setattr(modReport.data.cmsRun1, 'testVar', 'test01')

        report = Report()
        report.setStep(stepName='cmsRun1', stepSection=baseReport.retrieveStep('cmsRun1'))
        report.setStep(stepName='cmsRun1', stepSection=modReport.retrieveStep('cmsRun1'))

        self.assertEqual(report.listSteps(), ['cmsRun1'])
        self.assertEqual(report.data.cmsRun1.testVar, 'test01')

        return

    def testDeleteOutputModule(self):
        """
        _testDeleteOutputModule_

        If asked delete an output module, if it doesn't
        exist then do nothing
        """
        originalReport = Report("cmsRun1")
        originalReport.parse(self.xmlPath)

        self.assertTrue(originalReport.getOutputModule("cmsRun1", "outputALCARECORECO"),
                        "Error: Report XML doesn't have the module for the test, invalid test")

        originalOutputModules = len(originalReport.retrieveStep("cmsRun1").outputModules)
        originalReport.deleteOutputModuleForStep("cmsRun1", "outputALCARECORECO")
        self.assertFalse(originalReport.getOutputModule("cmsRun1", "outputALCARECORECO"),
                         "Error: The output module persists after deletion")
        self.assertEqual(len(originalReport.retrieveStep("cmsRun1").outputModules), originalOutputModules - 1,
                         "Error: The number of output modules is incorrect after deletion")

    def testSkippedFiles(self):
        """
        _testSkippedFiles_

        Test that skipped files are translated from FWJR into report
        """
        # Check a report where some files were skipped but not all
        originalReport = Report("cmsRun1")
        originalReport.parse(self.skippedFilesxmlPath)
        self.assertEqual(originalReport.getAllSkippedFiles(),
                         ['/store/data/Run2012D/Cosmics/RAW/v1/000/206/379/1ED243E7-A611-E211-A851-0019B9F581C9.root'])

        # For negative control, check a good report with no skipped files
        goodReport = Report("cmsRun1")
        goodReport.parse(self.xmlPath)
        self.assertEqual(goodReport.getAllSkippedFiles(), [])

        # Check a report where all files were skipped
        badReport = Report("cmsRun1")
        badReport.parse(self.skippedAllFilesxmlPath)
        self.assertEqual(sorted(badReport.getAllSkippedFiles()),
                         ['/store/data/Run2012D/Cosmics/RAW/v1/000/206/379/1ED243E7-A611-E211-A851-0019B9F581C9.root',
                          '/store/data/Run2012D/Cosmics/RAW/v1/000/206/379/1ED243E7-A622-E211-A851-0019B9F581C.root'])

        return

    def testSkippedFilesJSON(self):
        """
        _testSkippedFilesJSON_

        Test that skipped files are translated properly into JSON
        """
        # Check a report where some files were skipped but not all
        originalReport = Report("cmsRun1")
        originalReport.parse(self.skippedFilesxmlPath)
        originalJSON = originalReport.__to_json__(None)
        self.assertEqual(len(originalJSON['skippedFiles']), 1)

        # For negative control, check a good report with no skipped files
        goodReport = Report("cmsRun1")
        goodReport.parse(self.xmlPath)
        goodJSON = goodReport.__to_json__(None)
        self.assertEqual(goodJSON['skippedFiles'], [])

        # Check a report where all files were skipped
        badReport = Report("cmsRun1")
        badReport.parse(self.skippedAllFilesxmlPath)
        badJSON = badReport.__to_json__(None)
        self.assertEqual(len(badJSON['skippedFiles']), 2)

        return

    def testFallbackFiles(self):
        """
        _testFallbackFiles_

        Test that fallback files end up in the report
        """

        # For negative control, check a good report with no fallback files
        goodReport = Report("cmsRun1")
        goodReport.parse(self.xmlPath)
        self.assertEqual(goodReport.getAllFallbackFiles(), [])

        # Check a report where the file was a fallback
        badReport = Report("cmsRun1")
        badReport.parse(self.fallbackXmlPath)
        self.assertEqual(sorted(badReport.getAllFallbackFiles()),
                         [
                             '/store/data/Run2012D/SingleElectron/AOD/PromptReco-v1/000/207/279/D43A5B72-1831-E211-895D-001D09F24763.root'])

        twoReport = Report("cmsRun1")
        twoReport.parse(self.twoFileFallbackXmlPath)
        self.assertEqual(len(twoReport.getAllFallbackFiles()), 2)

        return

    def testPileupFiles(self):
        """
        _testPileupFiles_

        Test that alll the pileup files end up in the report
        """

        report = Report("cmsRun1")
        report.parse(self.pileupXmlPath)
        self.assertEqual(len(report.getAllInputFiles()), 14)

        primaryCount = 0
        secondaryCount = 0
        mixingCount = 0

        for fileEntry in report.getAllInputFiles():
            if fileEntry['input_type'] == 'mixingFiles':
                mixingCount += 1
            elif fileEntry['input_type'] == 'primaryFiles':
                primaryCount += 1
            elif fileEntry['input_type'] == 'secondaryFiles':
                secondaryCount += 1

        self.assertEqual(primaryCount, 1)
        self.assertEqual(secondaryCount, 0)
        self.assertEqual(mixingCount, 13)
        self.assertEqual(len(report.getAllFallbackFiles()), 1)

        return

    def testFallbackFilesJSON(self):
        """
        _testFallbackFilesJSON_

        Test that fallback attempt files are translated properly into JSON
        """

        # For negative control, check a good report with no skipped files
        goodReport = Report("cmsRun1")
        goodReport.parse(self.xmlPath)
        goodJSON = goodReport.__to_json__(None)
        self.assertEqual(goodJSON['fallbackFiles'], [])

        # Check a report where all files were skipped
        badReport = Report("cmsRun1")
        badReport.parse(self.fallbackXmlPath)
        badJSON = badReport.__to_json__(None)
        self.assertEqual(len(badJSON['fallbackFiles']), 1)

        return

    def testOutputCheck(self):
        """
        _testOutputCheck_

        Check that we can identify bad reports with no output files
        """
        badReport = Report("cmsRun1")
        badReport.parse(self.skippedAllFilesxmlPath)
        badReport.checkForOutputFiles("cmsRun1")
        self.assertFalse(badReport.stepSuccessful(stepName="cmsRun1"))
        self.assertEqual(badReport.getExitCode(), 60450)
        return

    def testNoLocationFile(self):
        """
        _testNoLocationFile_

        Check how we deal with output files without a valid location
        """
        jobReport = Report()
        jobReport.load(self.noLocationReport)
        fileList = jobReport.getAllFiles()
        self.assertEqual(len(fileList), 2)
        self.assertItemsEqual(fileList[0]['locations'], {})
        self.assertEqual(fileList[0]['outputModule'], "RAWSIMoutput")
        self.assertItemsEqual(fileList[1]['locations'], {"T2_CH_CSCS"})
        self.assertEqual(fileList[1]['outputModule'], "logArchive")


if __name__ == "__main__":
    unittest.main()



"""
_CreateWMBS_

Implementation of CreateWMBS for T0AST.

Inherit from Oracle.CreateWMBS, and add T0AST specific changes.
"""

from WMCore.WMBS.Oracle.Create import Create as OracleCreate


class CreateWMBS(OracleCreate):
    """
    Class to set up the WMBS schema in the T0AST database
    """

    def __init__(self, logger, dbInterface):
        """
        _init_

        Call the base class's constructor and create all necessary tables,
        constraints and inserts.
        """
        OracleCreate.__init__(self, logger, dbInterface)

        self.create["02wmbs_file_details"] = None  # This would be a view

        self.create["05wmbs_file_runlumi_map"] = None  # This would be a view

        for subType in ("Processing", "Merge", "Job"):
            subTypeQuery = "INSERT INTO wmbs_subs_type (name) values ('%s')" % subType
            self.inserts["wmbs_subs_type_%s" % subType] = subTypeQuery



from WMCore.Configuration import Configuration
config = Configuration()
config.section_('Agent')
# Agent:
config.Agent.hostName = None
config.Agent.contact = None
config.Agent.teamName = "team_usa"
config.Agent.agentName = None
config.section_('General')
# General: General Settings Section
config.General.workDir = '/home/test/application/WMAgentEmulator'
config.section_('CoreDatabase')
# CoreDatabase:
# dialect: Choose between oracle or mysql
# socket:  Set the socket file location for mysql (optional)
#
config.CoreDatabase.connectUrl='mysql://username:password@cmssrv18.fnal.gov:3306/TestDB'
config.component_('WMAgentEmulator')
# WMAgents:
config.WMAgentEmulator.componentDir = config.General.workDir + '/WMAgentEmulator'
config.WMAgentEmulator.namespace = "WMQuality.Emulators.WMAgents.WMAgentEmulator"
config.WMAgentEmulator.pollInterval = 10



#!/usr/bin/env python
"""
_AddFiles_

Oracle implementation of Jobs.AddFiles
"""

from WMCore.WMBS.MySQL.Jobs.AddFiles import AddFiles as AddFilesJobMySQL

class AddFiles(AddFilesJobMySQL):
    """
    _AddFiles_

    Oracle specific query: file is a reserved word
    """
    sql = """INSERT INTO wmbs_job_assoc (job, fileid)
               SELECT :jobid, :fileid FROM DUAL WHERE NOT EXISTS
                 (SELECT job FROM wmbs_job_assoc
                  WHERE job = :jobid AND fileid = :fileid)"""



#!/usr/bin/env python
"""
_WorkUnit.ExistsByTaskFileLumi_

MySQL implementation of WorkUnit.ExistsByTaskFileLumi
"""

from __future__ import absolute_import, division, print_function

from WMCore.Database.DBFormatter import DBFormatter


class ExistsByTaskFileLumi(DBFormatter):
    """
    _WorkUnit.ExistsByTaskFileLumi_

    MySQL implementation of WorkUnit.ExistsByTaskFileLumi
    """

    sql = ('SELECT wu.id as id FROM wmbs_workunit wu'
           ' INNER JOIN wmbs_frl_workunit_assoc assoc ON wu.id = assoc.workunit'
           ' WHERE wu.taskid=:taskid AND assoc.fileid=:fileid AND assoc.run=:run AND assoc.lumi=:lumi')

    def format(self, result):
        result = DBFormatter.format(self, result)
        if result:
            return result[0][0]
        else:
            return False

    def execute(self, taskid, fileid, run_lumi, conn=None, transaction=False):
        binds = {'taskid': taskid, 'fileid': fileid, 'run': run_lumi.run, 'lumi': run_lumi.lumis[0]}
        result = self.dbi.processData(self.sql, binds, conn=conn, transaction=transaction)
        return self.format(result)



#!/usr/bin/env python

from __future__ import division

from WMCore.WMBS.MySQL.Subscriptions.GetSubsWithoutJobGroup import GetSubsWithoutJobGroup as GetSubsWithoutJobGroupMySQL


class GetSubsWithoutJobGroup(GetSubsWithoutJobGroupMySQL):
    """ Same implementation as in MySQL"""
    pass
